# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.interpolate._bspl, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _colloc() -> typing.Any:
    'Build the B-spline collocation matrix.\n\n    The collocation matrix is defined as :math:`B_{j,l} = B_l(x_j)`,\n    so that row ``j`` contains all the B-splines which are non-zero\n    at ``x_j``.\n\n    The matrix is constructed in the LAPACK banded storage.\n    Basically, for an N-by-N matrix A with ku upper diagonals and\n    kl lower diagonals, the shape of the array Ab is (2*kl + ku +1, N),\n    where the last kl+ku+1 rows of Ab contain the diagonals of A, and\n    the first kl rows of Ab are not referenced.\n    For more info see, e.g. the docs for the ``*gbsv`` routine.\n\n    This routine is not supposed to be called directly, and\n    does no error checking.\n\n    Parameters\n    ----------\n    x : ndarray, shape (n,)\n        sorted 1D array of x values\n    t : ndarray, shape (nt + k + 1,)\n        sorted 1D array of knots\n    k : int\n        spline order\n    ab : ndarray, shape (2*kl + ku + 1, nt), F-order\n        This parameter is modified in-place.\n        On exit: zeroed out.\n        On exit: B-spline collocation matrix in the band storage with\n        ``ku`` upper diagonals and ``kl`` lower diagonals.\n        Here ``kl = ku = k``.\n    offset : int, optional\n        skip this many rows\n\n    '
    ...

def _handle_lhs_derivatives() -> typing.Any:
    ' Fill in the entries of the collocation matrix corresponding to known\n    derivatives at xval.\n\n    The collocation matrix is in the banded storage, as prepared by _colloc.\n    No error checking.\n\n    Parameters\n    ----------\n    t : ndarray, shape (nt + k + 1,)\n        knots\n    k : integer\n        B-spline order\n    xval : float\n        The value at which to evaluate the derivatives at.\n    ab : ndarray, shape(2*kl + ku + 1, nt), Fortran order\n        B-spline collocation matrix.\n        This argument is modified *in-place*.\n    kl : integer\n        Number of lower diagonals of ab.\n    ku : integer\n        Number of upper diagonals of ab.\n    deriv_ords : 1D ndarray\n        Orders of derivatives known at xval\n    offset : integer, optional\n        Skip this many rows of the matrix ab.\n\n    '
    ...

def _norm_eq_lsq(x, t, k, y, w, ab, rhs) -> typing.Any:
    'Construct the normal equations for the B-spline LSQ problem.\n\n    The observation equations are ``A @ c = y``, and the normal equations are\n    ``A.T @ A @ c = A.T @ y``. This routine fills in the rhs and lhs for the\n    latter.\n\n    The B-spline collocation matrix is defined as :math:`A_{j,l} = B_l(x_j)`,\n    so that row ``j`` contains all the B-splines which are non-zero\n    at ``x_j``.\n\n    The normal eq matrix has at most `2k+1` bands and is constructed in the\n    LAPACK symmetrix banded storage: ``A[i, j] == ab[i-j, j]`` with `i >= j`.\n    See the doctsring for `scipy.linalg.cholesky_banded` for more info.\n\n    This routine is not supposed to be called directly, and\n    does no error checking.\n\n    Parameters\n    ----------\n    x : ndarray, shape (n,)\n        sorted 1D array of x values\n    t : ndarray, shape (nt + k + 1,)\n        sorted 1D array of knots\n    k : int\n        spline order\n    y : ndarray, shape (n, s)\n        a 2D array of y values. The second dimension contains all trailing\n        dimensions of the original array of ordinates.\n    w : ndarray, shape(n,)\n        Weights.\n    ab : ndarray, shape (k+1, n), in Fortran order.\n        This parameter is modified in-place.\n        On entry: should be zeroed out.\n        On exit: LHS of the normal equations.\n    rhs : ndarray, shape (n, s), in Fortran order.\n        This parameter is modified in-place.\n        On entry: should be zeroed out.\n        On exit: RHS of the normal equations.\n\n    '
    ...

def evaluate_all_bspl() -> typing.Any:
    "Evaluate the ``k+1`` B-splines which are non-zero on interval ``m``.\n\n    Parameters\n    ----------\n    t : ndarray, shape (nt + k + 1,)\n        sorted 1D array of knots\n    k : int\n        spline order\n    xval: float\n        argument at which to evaluate the B-splines\n    m : int\n        index of the left edge of the evaluation interval, ``t[m] <= x < t[m+1]``\n    nu : int, optional\n        Evaluate derivatives order `nu`. Default is zero.\n\n    Returns\n    -------\n    ndarray, shape (k+1,)\n        The values of B-splines :math:`[B_{m-k}(xval), ..., B_{m}(xval)]` if\n        `nu` is zero, otherwise the derivatives of order `nu`.\n\n    Examples\n    --------\n\n    A textbook use of this sort of routine is plotting the ``k+1`` polynomial\n    pieces which make up a B-spline of order `k`.\n\n    Consider a cubic spline\n\n    >>> k = 3\n    >>> t = [0., 2., 2., 3., 4.]   # internal knots\n    >>> a, b = t[0], t[-1]    # base interval is [a, b)\n    >>> t = [a]*k + t + [b]*k  # add boundary knots\n\n    >>> import matplotlib.pyplot as plt\n    >>> xx = np.linspace(a, b, 100)\n    >>> plt.plot(xx, BSpline.basis_element(t[k:-k])(xx),\n    ...          'r-', lw=5, alpha=0.5)\n    >>> c = ['b', 'g', 'c', 'k']\n\n    Now we use slide an interval ``t[m]..t[m+1]`` along the base interval\n    ``a..b`` and use `evaluate_all_bspl` to compute the restriction of\n    the B-spline of interest to this interval:\n\n    >>> for i in range(k+1):\n    ...    x1, x2 = t[2*k - i], t[2*k - i + 1]\n    ...    xx = np.linspace(x1 - 0.5, x2 + 0.5)\n    ...    yy = [evaluate_all_bspl(t, k, x, 2*k - i)[i] for x in xx]\n    ...    plt.plot(xx, yy, c[i] + '--', lw=3, label=str(i))\n    ...\n    >>> plt.grid(True)\n    >>> plt.legend()\n    >>> plt.show()\n\n    "
    ...

def evaluate_spline(t, c, k, xp, nu, extrapolate, out) -> typing.Any:
    '\n    Evaluate a spline in the B-spline basis.\n\n    Parameters\n    ----------\n    t : ndarray, shape (n+k+1)\n        knots\n    c : ndarray, shape (n, m)\n        B-spline coefficients\n    xp : ndarray, shape (s,)\n        Points to evaluate the spline at.\n    nu : int\n        Order of derivative to evaluate.\n    extrapolate : int, optional\n        Whether to extrapolate to ouf-of-bounds points, or to return NaNs.\n    out : ndarray, shape (s, m)\n        Computed values of the spline at each of the input points.\n        This argument is modified in-place.\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Mapping
from numpy.typing import ArrayLike

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "extrema_bounding",
    "eccentricity",
    "diameter",
    "radius",
    "periphery",
    "center",
    "barycenter",
    "resistance_distance",
]

def extrema_bounding(G: Graph, compute="diameter"): ...
def eccentricity(G: Graph, v=None, sp=None) -> Mapping: ...
def diameter(G: Graph, e=None, usebounds=False): ...
def periphery(G: Graph, e=None, usebounds=False) -> ArrayLike: ...
def radius(G: Graph, e=None, usebounds=False): ...
def center(G: Graph, e=None, usebounds=False) -> ArrayLike: ...
def barycenter(G: Graph, weight=None, attr=None, sp=None) -> ArrayLike: ...
def resistance_distance(
    G: Graph, nodeA, nodeB, weight=None, invert_weight=True
) -> float: ...




from ._dispatcher import (
    BaseDistancesReductionDispatcher,
    ArgKmin,
    RadiusNeighbors,
    sqeuclidean_row_norms,
)

__all__ = [
    "BaseDistancesReductionDispatcher",
    "ArgKmin",
    "RadiusNeighbors",
    "sqeuclidean_row_norms",
]



from typing import (
    Any,
    Collection,
    Dict,
    Iterator,
    List,
    Mapping,
    NamedTuple,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
)

from django.db.models.base import Model
from django.db.models.fields import Field
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.sql.compiler import SQLCompiler
from django.db.models.sql.query import Query
from django.db.models.sql.where import WhereNode
from django.utils import tree

class PathInfo(NamedTuple):
    from_opts: Any
    to_opts: Any
    target_fields: Any
    join_field: Any
    m2m: Any
    direct: Any
    filtered_relation: Any

class InvalidQuery(Exception): ...

def subclasses(
    cls: Type[RegisterLookupMixin],
) -> Iterator[Type[RegisterLookupMixin]]: ...

class QueryWrapper:
    contains_aggregate: bool = ...
    data: Tuple[str, List[Any]] = ...
    def __init__(self, sql: str, params: List[Any]) -> None: ...
    def as_sql(self, compiler: SQLCompiler = ..., connection: Any = ...) -> Any: ...

class Q(tree.Node):
    AND: str = ...
    OR: str = ...
    conditional: bool = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __or__(self, other: Any) -> Q: ...
    def __and__(self, other: Any) -> Q: ...
    def __invert__(self) -> Q: ...
    def resolve_expression(
        self,
        query: Query = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> WhereNode: ...
    def deconstruct(self) -> Tuple[str, Tuple[Any, ...], Dict[str, str]]: ...

class DeferredAttribute:
    field_name: str = ...
    field: Field[Any, Any]
    def __init__(self, field_name: str) -> None: ...

class RegisterLookupMixin:
    lookup_name: str
    @classmethod
    def get_lookups(cls) -> Dict[str, Any]: ...
    def get_lookup(self, lookup_name: str) -> Optional[Any]: ...
    def get_transform(self, lookup_name: str) -> Optional[Any]: ...
    @staticmethod
    def merge_dicts(dicts: List[Dict[str, Any]]) -> Dict[str, Any]: ...
    @classmethod
    def register_lookup(
        cls, lookup: Any, lookup_name: Optional[str] = ...
    ) -> Type[Any]: ...
    @classmethod
    def _unregister_lookup(
        cls, lookup: Any, lookup_name: Optional[str] = ...
    ) -> Any: ...

def select_related_descend(
    field: Field[Any, Any],
    restricted: bool,
    requested: Optional[Mapping[str, Any]],
    load_fields: Optional[Collection[str]],
    reverse: bool = ...,
) -> bool: ...
def refs_expression(
    lookup_parts: Sequence[str], annotations: Mapping[str, bool]
) -> Tuple[bool, Sequence[str]]: ...
def check_rel_lookup_compatibility(
    model: Type[Model], target_opts: Any, field: FieldCacheMixin
) -> bool: ...

class FilteredRelation:
    relation_name: str = ...
    alias: Optional[str] = ...
    condition: Q = ...
    path: List[str] = ...
    def __init__(self, relation_name: str, *, condition: Any = ...) -> None: ...
    def clone(self) -> FilteredRelation: ...
    def resolve_expression(self, *args: Any, **kwargs: Any) -> None: ...
    def as_sql(self, compiler: SQLCompiler, connection: Any) -> Any: ...



from numpy.typing import ArrayLike
from heapq import heappop, heappush
from itertools import count

from ..classes.graph import Graph
from ..utils import not_implemented_for, pairwise

__all__ = [
    "all_simple_paths",
    "is_simple_path",
    "shortest_simple_paths",
    "all_simple_edge_paths",
]

def is_simple_path(G: Graph, nodes: ArrayLike) -> bool: ...
def all_simple_paths(G: Graph, source, target, cutoff=None): ...
def all_simple_edge_paths(G: Graph, source, target, cutoff=None): ...
def shortest_simple_paths(G: Graph, source, target, weight=None): ...

class PathBuffer:
    def __init__(self): ...
    def __len__(self): ...
    def push(self, cost, path): ...
    def pop(self): ...



from itertools import product

from ...classes.graph import Graph
from ...utils import not_implemented_for

__all__ = [
    "tensor_product",
    "cartesian_product",
    "lexicographic_product",
    "strong_product",
    "power",
    "rooted_product",
]

# Generators for producting graph products

def tensor_product(G: Graph, H): ...
def cartesian_product(G: Graph, H): ...
def lexicographic_product(G: Graph, H): ...
def strong_product(G: Graph, H): ...
def power(G: Graph, k): ...
def rooted_product(G: Graph, H, root): ...



from typing import Literal
from ..preprocessing import LabelBinarizer as LabelBinarizer
from ..utils._param_validation import (
    StrOptions as StrOptions,
    Interval as Interval,
    validate_params as validate_params,
)
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from numbers import Real as Real
from .._typing import MatrixLike, ArrayLike, Float
from ..utils.validation import (
    check_consistent_length as check_consistent_length,
    check_array as check_array,
)

import numpy as np


def l1_min_c(
    X: MatrixLike | ArrayLike,
    y: ArrayLike,
    *,
    loss: Literal["squared_hinge", "log", "squared_hinge"] = "squared_hinge",
    fit_intercept: bool = True,
    intercept_scaling: Float = 1.0
) -> Float:
    ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

__all__ = [
    "ModularProgram",
    "Function",
    "MainFunction",
    "Variable",
    "Varying",
    "FunctionChain",
    "Compiler",
    "MultiProgram",
]

from .program import ModularProgram as ModularProgram  # noqa
from .function import (
    Function as Function,
    MainFunction as MainFunction,
    FunctionChain as FunctionChain,
)  # noqa
from .function import StatementList as StatementList  # noqa
from .variable import Variable as Variable, Varying as Varying  # noqa
from .compiler import Compiler as Compiler  # noqa
from .multiprogram import MultiProgram as MultiProgram  # noqa



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._tools, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.csr as _mod_scipy_sparse_csr

DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def construct_dist_matrix(graph, predecessors, directed=..., null_value=...) -> typing.Any:
    '\n    construct_dist_matrix(graph, predecessors, directed=True, null_value=np.inf)\n\n    Construct distance matrix from a predecessor matrix\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    graph : array_like or sparse\n        The N x N matrix representation of a directed or undirected graph.\n        If dense, then non-edges are indicated by zeros or infinities.\n    predecessors : array_like\n        The N x N matrix of predecessors of each node (see Notes below).\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only move from\n        point i to point j along paths csgraph[i, j].\n        If False, then operate on an undirected graph: the algorithm can\n        progress from point i to j along csgraph[i, j] or csgraph[j, i].\n    null_value : bool, optional\n        value to use for distances between unconnected nodes.  Default is\n        np.inf\n\n    Returns\n    -------\n    dist_matrix : ndarray\n        The N x N matrix of distances between nodes along the path specified\n        by the predecessor matrix.  If no path exists, the distance is zero.\n\n    Notes\n    -----\n    The predecessor matrix is of the form returned by\n    `shortest_path`.  Row i of the predecessor matrix contains\n    information on the shortest paths from point i: each entry\n    predecessors[i, j] gives the index of the previous node in the path from\n    point i to point j.  If no path exists between point i and j, then\n    predecessors[i, j] = -9999\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import construct_dist_matrix\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 3)\t3\n\n    >>> pred = np.array([[-9999, 0, 0, 2],\n    ... [1, -9999, 0, 1],\n    ... [2, 0, -9999, 2],\n    ... [1, 3, 3, -9999]], dtype=np.int32)\n\n    >>> construct_dist_matrix(graph=graph, predecessors=pred, directed=False)\n    array([[ 0.,  1.,  2.,  5.],\n           [ 1.,  0.,  3.,  1.],\n           [ 2.,  3.,  0.,  3.],\n           [ 2.,  1.,  3.,  0.]])\n\n    '
    ...

def csgraph_from_dense(graph, null_value=..., nan_null=..., infinity_null=...) -> typing.Any:
    "\n    csgraph_from_dense(graph, null_value=0, nan_null=True, infinity_null=True)\n\n    Construct a CSR-format sparse graph from a dense matrix.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    graph : array_like\n        Input graph.  Shape should be (n_nodes, n_nodes).\n    null_value : float or None (optional)\n        Value that denotes non-edges in the graph.  Default is zero.\n    infinity_null : bool\n        If True (default), then infinite entries (both positive and negative)\n        are treated as null edges.\n    nan_null : bool\n        If True (default), then NaN entries are treated as non-edges\n\n    Returns\n    -------\n    csgraph : csr_matrix\n        Compressed sparse representation of graph,\n\n    Examples\n    --------\n    >>> from scipy.sparse.csgraph import csgraph_from_dense\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n\n    >>> csgraph_from_dense(graph)\n    <4x4 sparse matrix of type '<class 'numpy.float64'>'\n        with 4 stored elements in Compressed Sparse Row format>\n\n    "
    ...

def csgraph_from_masked(graph) -> typing.Any:
    "\n    csgraph_from_masked(graph)\n\n    Construct a CSR-format graph from a masked array.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    graph : MaskedArray\n        Input graph.  Shape should be (n_nodes, n_nodes).\n\n    Returns\n    -------\n    csgraph : csr_matrix\n        Compressed sparse representation of graph,\n\n    Examples\n    --------\n    >>> import numpy as np\n    >>> from scipy.sparse.csgraph import csgraph_from_masked\n\n    >>> graph_masked = np.ma.masked_array(data =[\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ...  ],\n    ... mask=[[ True, False, False , True],\n    ... [ True,  True , True, False],\n    ... [ True , True,  True ,False],\n    ... [ True ,True , True , True]],\n    ... fill_value = 0)\n\n    >>> csgraph_from_masked(graph_masked)\n    <4x4 sparse matrix of type '<class 'numpy.float64'>'\n        with 4 stored elements in Compressed Sparse Row format>\n\n    "
    ...

def csgraph_masked_from_dense(graph, null_value=..., nan_null=..., infinity_null=..., copy=...) -> typing.Any:
    '\n    csgraph_masked_from_dense(graph, null_value=0, nan_null=True,\n                              infinity_null=True, copy=True)\n\n    Construct a masked array graph representation from a dense matrix.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    graph : array_like\n        Input graph.  Shape should be (n_nodes, n_nodes).\n    null_value : float or None (optional)\n        Value that denotes non-edges in the graph.  Default is zero.\n    infinity_null : bool\n        If True (default), then infinite entries (both positive and negative)\n        are treated as null edges.\n    nan_null : bool\n        If True (default), then NaN entries are treated as non-edges\n\n    Returns\n    -------\n    csgraph : MaskedArray\n        masked array representation of graph\n\n    Examples\n    --------\n    >>> from scipy.sparse.csgraph import csgraph_masked_from_dense\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n\n    >>> csgraph_masked_from_dense(graph)\n    masked_array(\n      data=[[--, 1, 2, --],\n            [--, --, --, 1],\n            [--, --, --, 3],\n            [--, --, --, --]],\n      mask=[[ True, False, False,  True],\n            [ True,  True,  True, False],\n            [ True,  True,  True, False],\n            [ True,  True,  True,  True]],\n      fill_value=0)\n\n    '
    ...

def csgraph_to_dense(csgraph, null_value=...) -> typing.Any:
    "\n    csgraph_to_dense(csgraph, null_value=0)\n\n    Convert a sparse graph representation to a dense representation\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : csr_matrix, csc_matrix, or lil_matrix\n        Sparse representation of a graph.\n    null_value : float, optional\n        The value used to indicate null edges in the dense representation.\n        Default is 0.\n\n    Returns\n    -------\n    graph : ndarray\n        The dense representation of the sparse graph.\n\n    Notes\n    -----\n    For normal sparse graph representations, calling csgraph_to_dense with\n    null_value=0 produces an equivalent result to using dense format\n    conversions in the main sparse package.  When the sparse representations\n    have repeated values, however, the results will differ.  The tools in\n    scipy.sparse will add repeating values to obtain a final value.  This\n    function will select the minimum among repeating values to obtain a\n    final value.  For example, here we'll create a two-node directed sparse\n    graph with multiple edges from node 0 to node 1, of weights 2 and 3.\n    This illustrates the difference in behavior:\n\n    >>> from scipy.sparse import csr_matrix, csgraph\n    >>> data = np.array([2, 3])\n    >>> indices = np.array([1, 1])\n    >>> indptr = np.array([0, 2, 2])\n    >>> M = csr_matrix((data, indices, indptr), shape=(2, 2))\n    >>> M.toarray()\n    array([[0, 5],\n           [0, 0]])\n    >>> csgraph.csgraph_to_dense(M)\n    array([[0., 2.],\n           [0., 0.]])\n\n    The reason for this difference is to allow a compressed sparse graph to\n    represent multiple edges between any two nodes.  As most sparse graph\n    algorithms are concerned with the single lowest-cost edge between any\n    two nodes, the default scipy.sparse behavior of summming multiple weights\n    does not make sense in this context.\n\n    The other reason for using this routine is to allow for graphs with\n    zero-weight edges.  Let's look at the example of a two-node directed\n    graph, connected by an edge of weight zero:\n\n    >>> from scipy.sparse import csr_matrix, csgraph\n    >>> data = np.array([0.0])\n    >>> indices = np.array([1])\n    >>> indptr = np.array([0, 1, 1])\n    >>> M = csr_matrix((data, indices, indptr), shape=(2, 2))\n    >>> M.toarray()\n    array([[0, 0],\n           [0, 0]])\n    >>> csgraph.csgraph_to_dense(M, np.inf)\n    array([[ inf,   0.],\n           [ inf,  inf]])\n\n    In the first case, the zero-weight edge gets lost in the dense\n    representation.  In the second case, we can choose a different null value\n    and see the true form of the graph.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import csgraph_to_dense\n\n    >>> graph = csr_matrix( [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ])\n    >>> graph\n    <4x4 sparse matrix of type '<class 'numpy.int64'>'\n        with 4 stored elements in Compressed Sparse Row format>\n\n    >>> csgraph_to_dense(graph)\n    array([[ 0.,  1.,  2.,  0.],\n           [ 0.,  0.,  0.,  1.],\n           [ 0.,  0.,  0.,  3.],\n           [ 0.,  0.,  0.,  0.]])\n\n    "
    ...

def csgraph_to_masked(csgraph) -> typing.Any:
    "\n    csgraph_to_masked(csgraph)\n\n    Convert a sparse graph representation to a masked array representation\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : csr_matrix, csc_matrix, or lil_matrix\n        Sparse representation of a graph.\n\n    Returns\n    -------\n    graph : MaskedArray\n        The masked dense representation of the sparse graph.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import csgraph_to_masked\n\n    >>> graph = csr_matrix( [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ])\n    >>> graph\n    <4x4 sparse matrix of type '<class 'numpy.int64'>'\n        with 4 stored elements in Compressed Sparse Row format>\n\n    >>> csgraph_to_masked(graph)\n    masked_array(\n      data=[[--, 1.0, 2.0, --],\n            [--, --, --, 1.0],\n            [--, --, --, 3.0],\n            [--, --, --, --]],\n      mask=[[ True, False, False,  True],\n            [ True,  True,  True, False],\n            [ True,  True,  True, False],\n            [ True,  True,  True,  True]],\n      fill_value=1e+20)\n\n    "
    ...

csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def isspmatrix(x) -> typing.Any:
    'Is x of a sparse matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a sparse matrix\n\n    Returns\n    -------\n    bool\n        True if x is a sparse matrix, False otherwise\n\n    Notes\n    -----\n    issparse and isspmatrix are aliases for the same function.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix\n    >>> isspmatrix(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import isspmatrix\n    >>> isspmatrix(5)\n    False\n    '
    ...

def isspmatrix_csc(x) -> typing.Any:
    'Is x of csc_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csc matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csc matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csc_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csc_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_lil(x) -> typing.Any:
    'Is x of lil_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a lil matrix\n\n    Returns\n    -------\n    bool\n        True if x is a lil matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import lil_matrix, isspmatrix_lil\n    >>> isspmatrix_lil(lil_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import lil_matrix, csr_matrix, isspmatrix_lil\n    >>> isspmatrix_lil(csr_matrix([[5]]))\n    False\n    '
    ...

def reconstruct_path(csgraph, predecessors, directed=...) -> typing.Any:
    '\n    reconstruct_path(csgraph, predecessors, directed=True)\n\n    Construct a tree from a graph and a predecessor list.\n\n    .. versionadded:: 0.11.0\n\n    Parameters\n    ----------\n    csgraph : array_like or sparse matrix\n        The N x N matrix representing the directed or undirected graph\n        from which the predecessors are drawn.\n    predecessors : array_like, one dimension\n        The length-N array of indices of predecessors for the tree.  The\n        index of the parent of node i is given by predecessors[i].\n    directed : bool, optional\n        If True (default), then operate on a directed graph: only move from\n        point i to point j along paths csgraph[i, j].\n        If False, then operate on an undirected graph: the algorithm can\n        progress from point i to j along csgraph[i, j] or csgraph[j, i].\n\n    Returns\n    -------\n    cstree : csr matrix\n        The N x N directed compressed-sparse representation of the tree drawn\n        from csgraph which is encoded by the predecessor list.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import reconstruct_path\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [0, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 3)\t3\n\n    >>> pred = np.array([-9999, 0, 0, 1], dtype=np.int32)\n\n    >>> cstree = reconstruct_path(csgraph=graph, predecessors=pred, directed=False)\n    >>> cstree.todense()\n    matrix([[ 0.,  1.,  2.,  0.],\n            [ 0.,  0.,  0.,  1.],\n            [ 0.,  0.,  0.,  0.],\n            [ 0.,  0.,  0.,  0.]])\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------
from .svg import SVG as SVG
from .path import Path as Path  # noqa
from .base import namespace as namespace
from xml.etree import ElementTree as ElementTree

def Document(filename): ...



from typing import Mapping
from itertools import chain

from ...classes.graph import Graph

__all__ = ["tree_data", "tree_graph"]

# NOTE: Remove attrs from signature in 3.0
def tree_data(
    G: Graph,
    root,
    attrs: Mapping | None = None,
    ident: str = "id",
    children: str = "children",
) -> Mapping: ...
def tree_graph(
    data: Mapping,
    attrs: Mapping | None = None,
    ident: str = "id",
    children: str = "children",
): ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.application import catch_config_error
from IPython.core.crashhandler import CrashHandler
from IPython.core.application import BaseIPythonApplication
from IPython.core.shellapp import InteractiveShellApp

"""
The :class:`~traitlets.config.application.Application` object for the command
line :command:`ipython` program.
"""
_examples = ...
class IPAppCrashHandler(CrashHandler):
    """sys.excepthook for IPython itself, leaves a detailed report on disk."""
    def __init__(self, app) -> None:
        ...
    
    def make_report(self, traceback):
        """Return a string containing a crash report."""
        ...
    


flags = ...
frontend_flags = ...
addflag = ...
classic_config = ...
aliases = ...
class LocateIPythonApp(BaseIPythonApplication):
    description = ...
    subcommands = ...
    def start(self): # -> None:
        ...
    


class TerminalIPythonApp(BaseIPythonApplication, InteractiveShellApp):
    name = ...
    description = ...
    crash_handler_class = IPAppCrashHandler
    examples = ...
    flags = ...
    aliases = ...
    classes = ...
    interactive_shell_class = ...
    subcommands = ...
    auto_create = ...
    quick = ...
    display_banner = ...
    force_interact = ...
    something_to_run = ...
    @catch_config_error
    def initialize(self, argv=...): # -> None:
        """Do actions after construct, but before starting the app."""
        ...
    
    def init_shell(self): # -> None:
        """initialize the InteractiveShell instance"""
        ...
    
    def init_banner(self): # -> None:
        """optionally display the banner"""
        ...
    
    def start(self): # -> None:
        ...
    


def load_default_config(ipython_dir=...): # -> Instance:
    """Load the default config file from the default ipython_dir.

    This is useful for embedded shells.
    """
    ...

launch_new_instance = ...
if __name__ == '__main__':
    ...



from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)

from django.db.models.fields import CharField, EmailField, TextField
from typing_extensions import Literal

_Choice = Tuple[Any, Any]
_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]

_ValidatorCallable = Callable[..., None]
_ErrorMessagesToOverride = Dict[str, Any]

_C = TypeVar("_C", bound="Optional[str]")

class CIText: ...

class CICharField(CIText, CharField[_C]):
    @overload
    def __init__(
        self: CICharField[str],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: CICharField[Optional[str]],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    def __get__(self: CICharField[_C], instance: Any, owner: Any) -> _C: ...
    def __set__(self: CICharField[_C], instance: Any, value: _C) -> None: ...  # type: ignore [override]

class CIEmailField(CIText, EmailField[_C]):
    @overload
    def __init__(
        self: CIEmailField[str],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: CIEmailField[Optional[str]],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    def __get__(self: CIEmailField[_C], instance: Any, owner: Any) -> _C: ...
    def __set__(self, instance: Any, value: _C) -> None: ...  # type: ignore [override]

class CITextField(CIText, TextField[_C]):
    @overload
    def __init__(
        self: CITextField[str],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: CITextField[Optional[str]],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    def __get__(self: CITextField[_C], instance: Any, owner: Any) -> _C: ...
    def __set__(self: CITextField[_C], instance: Any, value: _C) -> None: ...  # type: ignore [override]



from collections import deque

from ...classes.graph import Graph

__all__ = [
    "CurrentEdge",
    "Level",
    "GlobalRelabelThreshold",
    "build_residual_network",
    "detect_unboundedness",
    "build_flow_dict",
]

class CurrentEdge:

    __slots__ = ...

    def __init__(self, edges): ...
    def get(self): ...
    def move_to_next(self): ...
    def _rewind(self): ...

class Level:

    __slots__ = ...

    def __init__(self): ...

class GlobalRelabelThreshold:
    def __init__(self, n, m, freq): ...
    def add_work(self, work): ...
    def is_reached(self): ...
    def clear_work(self): ...

def build_residual_network(G: Graph, capacity): ...
def detect_unboundedness(R, s, t): ...
def build_flow_dict(G: Graph, R): ...



import datetime
from re import L
from matplotlib.backend_tools import Cursors
from matplotlib.contour import QuadContourSet
import numpy as np
from typing import Callable, Literal, Sequence, overload
from matplotlib._typing import *
from matplotlib.transforms import Bbox, BboxTransformTo, Transform
from matplotlib.text import Annotation, Text
from matplotlib.container import BarContainer, ErrorbarContainer, StemContainer
from matplotlib.patches import FancyArrow, Polygon, Rectangle, StepPatch, Wedge
from matplotlib.quiver import Barbs, Quiver
from matplotlib.colors import Colormap, Normalize
from matplotlib.axes._secondary_axes import SecondaryAxis
from matplotlib.collections import (
    EventCollection,
    LineCollection,
    BrokenBarHCollection,
    PathCollection,
    PolyCollection,
    Collection,
    QuadMesh,
)
from matplotlib.lines import Line2D
from matplotlib.legend import Legend
from matplotlib.markers import MarkerStyle
from matplotlib.ticker import Formatter
from matplotlib.figure import Figure
from matplotlib.image import AxesImage, PcolorImage
from matplotlib.axis import XAxis, YAxis
from matplotlib.artist import Artist
from matplotlib.spines import Spines
from ._base import _AxesBase

from matplotlib.table import table as table_table
from matplotlib.tri import \
    tricontour as tri_tricontour, \
    tricontour as tri_tricontourf, \
    tripcolor as tri_tripcolor, \
    triplot as tri_triplot

    
class Axes(_AxesBase):

    dataLim: Bbox
    viewLim: Bbox
    figure: Figure
    transAxes: BboxTransformTo
    transData: Transform
    xaxis: XAxis
    yaxis: YAxis
    spines: Spines
    fmt_xdata: None | Formatter = ...
    fmt_ydata: None | Formatter = ...
    cursor_to_use: Cursors = ...

    def get_title(self, loc: Literal["center", "left", "right"] = ...) -> str: ...
    def set_title(
        self,
        label: str,
        fontdict: dict = ...,
        loc: Literal["center", "left", "right"] = ...,
        pad: float = ...,
        *,
        y: float = ...,
        **kwargs
    ) -> Text: ...
    def get_legend_handles_labels(self, legend_handler_map=...) -> tuple[list, list]: ...
    def legend(self, *args, **kwargs) -> Legend: ...
    def inset_axes(
        self,
        bounds: Sequence[float],
        *,
        transform: Transform = ...,
        zorder: float = ...,
        **kwargs
    ) -> Axes: ...
    def indicate_inset(
        self,
        bounds: Sequence[float],
        inset_ax: Axes = ...,
        *,
        transform: Transform = ...,
        facecolor: Color = ...,
        edgecolor: Color = ...,
        alpha: float = ...,
        zorder: float = ...,
        **kwargs
    ) -> Rectangle: ...
    def indicate_inset_zoom(self, inset_ax: Axes, **kwargs) -> Rectangle: ...
    def secondary_xaxis(
        self,
        location: Literal["top", "bottom", "left", "right"] | float,
        *,
        functions=...,
        **kwargs
    ) -> SecondaryAxis: ...
    def secondary_yaxis(
        self,
        location: Literal["top", "bottom", "left", "right"] | float,
        *,
        functions=...,
        **kwargs
    ) -> SecondaryAxis: ...
    def text(
        self, x: float, y: float, s: str, fontdict: dict = ..., **kwargs
    ) -> Text: ...
    def annotate(
        self,
        text: str,
        xy: Sequence[float],
        xytext: Sequence[float] = ...,
        xycoords: str | Artist | Transform | Callable = ...,
        textcoords: str | Artist | Transform | Callable = ...,
        arrowprops: dict = ...,
        annotation_clip: bool | None = ...,
        **kwargs
    ) -> Annotation: ...
    def axhline(
        self, y: float = 0, xmin: float = 0, xmax: float = 1, **kwargs
    ) -> Line2D: ...
    def axvline(
        self, x: float = ..., ymin: float = ..., ymax: float = ..., **kwargs
    ) -> Line2D: ...
    def axline(
        self,
        xy1: tuple[float, float],
        xy2: tuple[float, float] = ...,
        *,
        slope: float = ...,
        **kwargs
    ) -> Line2D: ...
    def axhspan(
        self, ymin: float, ymax: float, xmin: float = ..., xmax: float = ..., **kwargs
    ) -> Polygon: ...
    def axvspan(
        self, xmin: float, xmax: float, ymin: float = 0, ymax: float = 1, **kwargs
    ) -> Polygon: ...
    def hlines(
        self,
        y: float | ArrayLike,
        xmin: float | ArrayLike,
        xmax: float | ArrayLike,
        colors: list[Color] = ...,
        linestyles: Literal["solid", "dashed", "dashdot", "dotted"] = ...,
        label: str = ...,
        **kwargs
    ) -> LineCollection: ...
    def vlines(
        self,
        x: float | ArrayLike,
        ymin: float | ArrayLike,
        ymax: float | ArrayLike,
        colors: list[Color] = ...,
        linestyles: Literal["solid", "dashed", "dashdot", "dotted"] = ...,
        label: str = ...,
        **kwargs
    ) -> LineCollection: ...
    def eventplot(
        self,
        positions: ArrayLike | list[ArrayLike],
        orientation: Literal["horizontal", "vertical"] = "horizontal",
        lineoffsets: float | ArrayLike = 1,
        linelengths: float | ArrayLike = 1,
        linewidths: float | ArrayLike = ...,
        colors: Color | list[Color] = ...,
        linestyles: str | tuple | list = ...,
        **kwargs
    ) -> list[EventCollection]: ...
    def plot(
        self, *args, scalex=..., scaley=..., data=..., **kwargs
    ) -> list[Line2D]: ...
    def plot_date(
        self,
        x: ArrayLike,
        y: ArrayLike,
        fmt: str = ...,
        tz: datetime.tzinfo = ...,
        xdate: bool = ...,
        ydate: bool = ...,
        **kwargs
    ) -> list[Line2D]: ...
    def loglog(self, *args, **kwargs) -> list[Line2D]: ...
    def semilogx(self, *args, **kwargs) -> list[Line2D]: ...
    def semilogy(self, *args, **kwargs) -> list[Line2D]: ...
    def acorr(self, x: ArrayLike, **kwargs): ...
    def xcorr(
        self,
        x,
        y,
        normed: bool = True,
        detrend: Callable = ...,
        usevlines: bool = True,
        maxlags: int = 10,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, bool, int]: ...
    def step(
        self,
        x: ArrayLike,
        y: ArrayLike,
        *args,
        where: Literal["pre", "post", "mid"] = ...,
        data=...,
        **kwargs
    ) -> list[Line2D]: ...
    def bar(
        self,
        x: float | ArrayLike,
        height: float | ArrayLike,
        width: float | ArrayLike = ...,
        bottom: float | ArrayLike = ...,
        *,
        align: Literal["center", "edge"] = "center",
        **kwargs
    ) -> BarContainer: ...
    def barh(
        self,
        y: float | ArrayLike,
        width: float | ArrayLike,
        height: float | ArrayLike = ...,
        left: float | ArrayLike = ...,
        *,
        align: Literal["center", "edge"] = "center",
        **kwargs
    ) -> BarContainer: ...
    def bar_label(
        self,
        container: BarContainer,
        labels: ArrayLike = ...,
        *,
        fmt: str = "%g",
        label_type: Literal["edge", "center"] = "edge",
        padding: float = 0,
        **kwargs
    ) -> list[Text]: ...
    def broken_barh(
        self,
        xranges: Sequence[tuple[float, float]],
        yrange: tuple[float, float],
        **kwargs
    ) -> BrokenBarHCollection: ...
    def stem(
        self,
        *args,
        linefmt: str = ...,
        markerfmt: str = ...,
        basefmt: str = "C3-",
        bottom: float = 0,
        label: str | None = None,
        use_line_collection: bool = True,
        orientation: str = "verical"
    ) -> StemContainer: ...
    def pie(
        self,
        x,
        explode: ArrayLike | None = None,
        labels: list | None = None,
        colors: ArrayLike | None = None,
        autopct: None | str | Callable = None,
        pctdistance: float = 0.6,
        shadow: bool = False,
        labeldistance: float | None = 1.1,
        startangle: float = 0,
        radius: float = 1,
        counterclock: bool = True,
        wedgeprops: dict | None = None,
        textprops: dict | None = None,
        center: tuple[float, float] = (0, 0),
        frame: bool = False,
        rotatelabels: bool = False,
        *,
        normalize: bool = True
    ) -> tuple[list[Wedge], list[Text], list[Text]]: ...
    def errorbar(
        self,
        x: float | ArrayLike,
        y: float | ArrayLike,
        yerr: float | ArrayLike = ...,
        xerr: float | ArrayLike = ...,
        fmt: str = "",
        ecolor: Color | None = None,
        elinewidth: float | None = None,
        capsize: float = ...,
        barsabove: bool = False,
        lolims: bool = False,
        uplims: bool = False,
        xlolims: bool = False,
        xuplims: bool = False,
        errorevery: int = 1,
        capthick: float | None = None,
        **kwargs
    ) -> ErrorbarContainer: ...
    def boxplot(
        self,
        x: ArrayLike,
        notch: bool = False,
        sym: str = ...,
        vert: bool = True,
        whis: float = 1.5,
        positions: ArrayLike = ...,
        widths: float | ArrayLike = ...,
        patch_artist: bool = False,
        bootstrap: int = ...,
        usermedians=...,
        conf_intervals: ArrayLike = ...,
        meanline: bool = False,
        showmeans=...,
        showcaps=...,
        showbox=...,
        showfliers=...,
        boxprops=...,
        labels: Sequence = ...,
        flierprops=...,
        medianprops=...,
        meanprops=...,
        capprops=...,
        whiskerprops=...,
        manage_ticks: bool = True,
        autorange: bool = False,
        zorder: float = 2,
        capwidths=...,
    ) -> dict[str, list[Line2D]]: ...
    def bxp(
        self,
        bxpstats: list[dict],
        positions: ArrayLike = ...,
        widths: float | ArrayLike | None = None,
        vert: bool = True,
        patch_artist: bool = False,
        shownotches: bool = False,
        showmeans: bool = False,
        showcaps: bool = True,
        showbox: bool = True,
        showfliers: bool = True,
        boxprops: dict = ...,
        whiskerprops: dict = ...,
        flierprops: dict = ...,
        medianprops: dict = ...,
        capprops: dict = ...,
        meanprops: dict = ...,
        meanline: bool = False,
        manage_ticks: bool = True,
        zorder: float = 2,
        capwidths: float | ArrayLike | None = None,
    ) -> dict[str, list[Line2D]]: ...
    def scatter(
        self,
        x: float | ArrayLike,
        y: float | ArrayLike,
        s: float | ArrayLike = ...,
        c: ArrayLike | list[Color] | Color = ...,
        marker: MarkerStyle = ...,
        cmap: str | Colormap = ...,
        norm: Normalize | None = None,
        vmin: float | None = None,
        vmax: float | None = None,
        alpha: float | None = None,
        linewidths: float | ArrayLike = ...,
        *,
        edgecolors: Color = ...,
        plotnonfinite: bool = False,
        **kwargs
    ) -> PathCollection: ...
    def hexbin(
        self,
        x: ArrayLike,
        y: ArrayLike,
        C: ArrayLike = ...,
        gridsize: int = 100,
        bins: Literal["log"] | int | Sequence | None = None,
        xscale: Literal["linear", "log"] = "linear",
        yscale: Literal["linear", "log"] = "linear",
        extent: Sequence[float] | None = None,
        cmap=...,
        norm=...,
        vmin=...,
        vmax=...,
        alpha=...,
        linewidths=...,
        edgecolors=...,
        reduce_C_function=...,
        mincnt: int | None = None,
        marginals: bool = False,
        **kwargs
    ) -> PolyCollection: ...
    def arrow(
        self, x: float, y: float, dx: float, dy: float, **kwargs
    ) -> FancyArrow: ...
    def quiverkey(
        self, Q: Quiver, X: float, Y: float, U: float, label: str, **kwargs
    ): ...
    def quiver(self, *args, **kwargs) -> Quiver: ...
    def barbs(self, *args, **kwargs) -> Barbs: ...
    def fill(self, *args, data=..., **kwargs) -> list[Polygon]: ...
    def fill_between(
        self,
        x,
        y1: Scalar,
        y2: Scalar = ...,
        where: ArrayLike = ...,
        interpolate: bool = ...,
        step: Literal["pre", "post", "mid"] = ...,
        **kwargs
    ) -> PolyCollection: ...
    def fill_betweenx(
        self,
        y,
        x1: Scalar,
        x2: Scalar = ...,
        where: ArrayLike = ...,
        step: Literal["pre", "post", "mid"] = ...,
        interpolate: bool = ...,
        **kwargs
    ) -> PolyCollection: ...
    def imshow(
        self,
        X: ArrayLike,
        cmap: str | Colormap = ...,
        norm: Normalize = ...,
        aspect: Literal["equal", "auto"] | float = ...,
        interpolation: str = ...,
        alpha: float | ArrayLike = ...,
        vmin: float = ...,
        vmax: float = ...,
        origin: Literal["upper", "lower"] = ...,
        extent: Sequence[float] = ...,
        *,
        interpolation_stage: Literal["data", "rgba"] = ...,
        filternorm: bool = True,
        filterrad: float = 4,
        resample: bool = ...,
        url: str = ...,
        **kwargs
    ) -> AxesImage: ...
    def pcolor(
        self,
        *args,
        shading: Literal["flat", "nearest", "auto"] = ...,
        alpha: float | None = None,
        norm: Normalize = ...,
        cmap: str | Colormap = ...,
        vmin: float | None = None,
        vmax: float | None = None,
        **kwargs
    ) -> Collection: ...
    def pcolormesh(
        self,
        *args,
        alpha: float | None = None,
        norm: Normalize = ...,
        cmap: str | Colormap = ...,
        vmin: float | None = None,
        vmax: float | None = None,
        shading: Literal["flat", "nearest", "gouraud", "auto"] = ...,
        antialiased=...,
        **kwargs
    ) -> QuadMesh: ...
    def pcolorfast(
        self,
        *args,
        alpha: float | None = None,
        norm: Normalize = ...,
        cmap: str | Colormap = ...,
        vmin: float | None = None,
        vmax: float | None = None,
        **kwargs
    ) -> tuple[AxesImage, PcolorImage, QuadMesh]: ...
    def contour(self, *args, **kwargs) -> QuadContourSet: ...
    def contourf(self, *args, **kwargs) -> QuadContourSet: ...
    def clabel(self, CS, levels: ArrayLike = ..., **kwargs): ...
    @overload
    def hist(
        self,
        x: Sequence[ArrayLike],
        bins: int | ArrayLike | str = ...,
        range: tuple | None = ...,
        density: bool = ...,
        weights: None = ...,
        cumulative: bool | Literal[-1] = ...,
        bottom=...,
        histtype: Literal["bar", "barstacked", "step", "stepfilled"] = ...,
        align: Literal["left", "mid", "right"] = ...,
        orientation: Literal["vertical", "horizontal"] = ...,
        rwidth: float | None = ...,
        log: bool = ...,
        color: Color | None = ...,
        label: str | None = ...,
        stacked: bool = ...,
        **kwargs
    ) -> tuple[list[list[float]], list[float], BarContainer | list]: ...
    @overload
    def hist(
        self,
        x: ArrayLike,
        bins: int | ArrayLike | str = ...,
        range: tuple[float, float] | None = None,
        density: bool = False,
        weights: ArrayLike | None = None,
        cumulative: bool | Literal[-1] = False,
        bottom: ArrayLike | Scalar | None = None,
        histtype: Literal["bar", "barstacked", "step", "stepfilled"] = "bar",
        align: Literal["left", "mid", "right"] = "mid",
        orientation: Literal["vertical", "horizontal"] = "vertical",
        rwidth: float | None = None,
        log: bool = False,
        color: Color | None = None,
        label: str | None = None,
        stacked: bool = False,
        **kwargs
    ) -> tuple[list[float], list[float], BarContainer | list]: ...
    def stairs(
        self,
        values: ArrayLike,
        edges: ArrayLike = ...,
        *,
        orientation: Literal["vertical", "horizontal"] = "vertical",
        baseline: float | ArrayLike | None = 0,
        fill: bool = False,
        **kwargs
    ) -> StepPatch: ...
    def hist2d(
        self,
        x,
        y,
        bins: None | int | ArrayLike = ...,
        range=...,
        density: bool = False,
        weights=...,
        cmin: float | None = None,
        cmax: float | None = None,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, np.ndarray, tuple[float, float]|None]: ...
    def psd(
        self,
        x: Sequence,
        NFFT: int = ...,
        Fs: float = ...,
        Fc: int = 0,
        detrend: Literal["none", "mean", "linear"] | Callable = ...,
        window: Callable | np.ndarray = ...,
        noverlap: int = 0,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        scale_by_freq: bool = ...,
        return_line: bool = False,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, Line2D]: ...
    def csd(
        self,
        x: ArrayLike,
        y: ArrayLike,
        NFFT: int = ...,
        Fs: float = ...,
        Fc: int = 0,
        detrend: Literal["none", "mean", "linear"] | Callable = ...,
        window: Callable | np.ndarray = ...,
        noverlap: int = 0,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        scale_by_freq: bool = ...,
        return_line: bool = False,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, Line2D]: ...
    def magnitude_spectrum(
        self,
        x: Sequence,
        Fs: float = ...,
        Fc: int = ...,
        window: Callable | np.ndarray = ...,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        scale: Literal["default", "linear", "dB"] = "linear",
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, Line2D]: ...
    def angle_spectrum(
        self,
        x: Sequence,
        Fs: float = ...,
        Fc: int = 0,
        window: Callable | np.ndarray = ...,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, Line2D]: ...
    def phase_spectrum(
        self,
        x: Sequence,
        Fs: float = ...,
        Fc: int = 0,
        window: Callable | np.ndarray = ...,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, Line2D]: ...
    def cohere(
        self,
        x,
        y,
        NFFT: int = ...,
        Fs: float = ...,
        Fc: int = 0,
        detrend: Literal["none", "mean", "linear"] | Callable = ...,
        window: Callable | np.ndarray = ...,
        noverlap: int = 0,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        scale_by_freq: bool = ...,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray]: ...
    def specgram(
        self,
        x: Sequence,
        NFFT: int = ...,
        Fs: float = ...,
        Fc: int = 0,
        detrend: Literal["none", "mean", "linear"] | Callable = ...,
        window: Callable | np.ndarray = ...,
        noverlap: int = 128,
        cmap: Colormap = ...,
        xextent=...,
        pad_to: int = ...,
        sides: Literal["default", "onesided", "twosided"] = ...,
        scale_by_freq: bool = ...,
        mode: Literal["default", "psd", "magnitude", "angle", "phase"] = "psd",
        scale: Literal["default", "linear", "dB"] = "dB",
        vmin=...,
        vmax=...,
        **kwargs
    ) -> tuple[np.ndarray, np.ndarray, np.ndarray, AxesImage]: ...
    def spy(
        self,
        Z,
        precision: float | Literal["present"] = 0,
        marker=...,
        markersize=...,
        aspect: Literal["equal", "auto", None] | float = "equal",
        origin: Literal["upper", "lower"] = ...,
        **kwargs
    ) -> AxesImage | Line2D: ...
    def matshow(self, Z: ArrayLike, **kwargs) -> AxesImage: ...
    def violinplot(
        self,
        dataset: ArrayLike,
        positions: ArrayLike = ...,
        vert: bool = True,
        widths: ArrayLike | float = 0.5,
        showmeans: bool = False,
        showextrema: bool = True,
        showmedians: bool = False,
        quantiles: ArrayLike | None = None,
        points: int = 100,
        bw_method: str | Scalar | Callable | None = None,
    ) -> dict[str, Collection]: ...
    def violin(
        self,
        vpstats: list[dict],
        positions: ArrayLike = ...,
        vert: bool = True,
        widths: ArrayLike | float = 0.5,
        showmeans: bool = False,
        showextrema: bool = True,
        showmedians: bool = False,
    ) -> dict[str, Collection]: ...
    table = table_table
    stackplot = ...
    streamplot = ...
    tricontour = tri_tricontour.tricontour
    tricontourf = tri_tricontourf.tricontourf
    tripcolor = tri_tripcolor.tripcolor
    triplot = tri_triplot.triplot



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: _odrpack, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def _set_exceptions(odr_error, odr_stop) -> typing.Any:
    '_set_exceptions(odr_error, odr_stop)\n\n    Internal function: set exception classes.'
    ...

def odr(fcn, beta0, y, x, we=..., wd=..., fjacb=..., fjacd=..., extra_args=..., ifixx=..., ifixb=..., job=..., iprint=..., errfile=..., rptfile=..., ndigit=..., taufac=..., sstol=..., partol=..., maxit=..., stpb=..., stpd=..., sclb=..., scld=..., work=..., iwork=..., full_output=...) -> typing.Any:
    'odr(fcn, beta0, y, x, we=None, wd=None, fjacb=None, fjacd=None, extra_args=None, ifixx=None, ifixb=None, job=0, iprint=0, errfile=None, rptfile=None, ndigit=0, taufac=0.0, sstol=-1.0, partol=-1.0, maxit=-1, stpb=None, stpd=None, sclb=None, scld=None, work=None, iwork=None, full_output=0)\n\n    Low-level function for ODR.\n\n    See Also\n    --------\n    ODR : The ODR class gathers all information and coordinates the running of the main fitting routine.\n    Model : The Model class stores information about the function you wish to fit.\n    Data : The data to fit.\n    RealData : Data with weights as actual std. dev.s and/or covariances.\n\n    Notes\n    -----\n    This is a function performing the same operation as the `ODR`,\n    `Model`, and `Data` classes together. The parameters of this\n    function are explained in the class documentation.'
    ...

def __getattr__(name) -> typing.Any:
    ...




# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._trlib._trlib, version: unspecified
import typing
import builtins as _mod_builtins
import scipy.optimize._trustregion as _mod_scipy_optimize__trustregion

BaseQuadraticSubproblem = _mod_scipy_optimize__trustregion.BaseQuadraticSubproblem
class TRLIBQuadraticSubproblem(_mod_scipy_optimize__trustregion.BaseQuadraticSubproblem):
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, x, fun, jac, hess, hessp, tol_rel_i, tol_rel_b, disp) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def solve(self, trust_radius) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _minimize_trust_region(fun, x0, args, jac, hess, hessp, subproblem, initial_trust_radius, max_trust_radius, eta, gtol, maxiter, disp, return_all, callback, inexact, **unknown_options) -> typing.Any:
    '\n    Minimization of scalar function of one or more variables using a\n    trust-region algorithm.\n\n    Options for the trust-region algorithm are:\n        initial_trust_radius : float\n            Initial trust radius.\n        max_trust_radius : float\n            Never propose steps that are longer than this value.\n        eta : float\n            Trust region related acceptance stringency for proposed steps.\n        gtol : float\n            Gradient norm must be less than `gtol`\n            before successful termination.\n        maxiter : int\n            Maximum number of iterations to perform.\n        disp : bool\n            If True, print convergence message.\n        inexact : bool\n            Accuracy to solve subproblems. If True requires less nonlinear\n            iterations, but more vector products. Only effective for method\n            trust-krylov.\n\n    This function is called by the `minimize` function.\n    It is not supposed to be called directly.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from itertools import combinations

from ..classes.graph import Graph

from ..utils import arbitrary_element, not_implemented_for

__all__ = [
    "is_eulerian",
    "eulerian_circuit",
    "eulerize",
    "is_semieulerian",
    "has_eulerian_path",
    "eulerian_path",
]

def is_eulerian(G: Graph): ...
def is_semieulerian(G: Graph): ...
def eulerian_circuit(G: Graph, source=None, keys: bool = False): ...
def has_eulerian_path(G: Graph, source=None): ...
def eulerian_path(G: Graph, source=None, keys=False): ...
def eulerize(G: Graph): ...



"""Startup for Python.

Avoid activating Rich features that break functionality outside of the REPL.
"""

from warnings import resetwarnings
from warning_filters import filter_warnings_and_update_dotenv


def init():
    from rich import inspect, traceback  # noqa: F401

    # Reset warnings coming from `.env` since those in `warning_filters.py` are smarter
    resetwarnings()
    filter_warnings_and_update_dotenv()

    traceback.install()

    if not is_notebook_or_ipython():
        from rich import pretty, print  # noqa: F401

        pretty.install()


# https://stackoverflow.com/a/39662359
def is_notebook_or_ipython() -> bool:
    try:
        shell = get_ipython().__class__.__name__  # type: ignore  # pyright 1.1.308, dynamic
    except NameError:
        return False  # Probably standard Python interpreter
    else:
        return shell == "TerminalInteractiveShell"


init()



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""Magic functions for InteractiveShell.
"""
magics = ...
magic_kinds = ...
magic_spec = ...
magic_escapes = ...
class Bunch:
    ...


def on_off(tag):
    """Return an ON/OFF string for a 1/0 input. Simple utility function."""
    ...

def compress_dhist(dh):
    """Compress a directory history into a new one with at most 20 entries.

    Return a new list made from the first and last 10 elements of dhist after
    removal of duplicates.
    """
    ...

def needs_local_scope(func):
    """Decorator to mark magic functions which need to local scope to run."""
    ...

def magics_class(cls):
    """Class decorator for all subclasses of the main Magics class.

    Any class that subclasses Magics *must* also apply this decorator, to
    ensure that all the methods that have been decorated as line/cell magics
    get correctly registered in the class instance.  This is necessary because
    when method decorators run, the class does not exist yet, so they
    temporarily store their information into a module global.  Application of
    this class decorator copies that global data to the class instance and
    clears the global.

    Obviously, this mechanism is not thread-safe, which means that the
    *creation* of subclasses of Magic should only be done in a single-thread
    context.  Instantiation of the classes has no restrictions.  Given that
    these classes are typically created at IPython startup time and before user
    application code becomes active, in practice this should not pose any
    problems.
    """
    ...

def record_magic(dct, magic_kind, magic_name, func): # -> None:
    """Utility function to store a function as a magic of a specific kind.

    Parameters
    ----------
    dct : dict
        A dictionary with 'line' and 'cell' subdicts.
    magic_kind : str
        Kind of magic to be stored.
    magic_name : str
        Key to store the magic as.
    func : function
        Callable object to store.
    """
    ...

def validate_type(magic_kind): # -> None:
    """Ensure that the given magic_kind is valid.

    Check that the given magic_kind is one of the accepted spec types (stored
    in the global `magic_spec`), raise ValueError otherwise.
    """
    ...

_docstring_template = ...
MAGIC_NO_VAR_EXPAND_ATTR = ...
MAGIC_OUTPUT_CAN_BE_SILENCED = ...
def no_var_expand(magic_func):
    """Mark a magic function as not needing variable expansion

    By default, IPython interprets `{a}` or `$a` in the line passed to magics
    as variables that should be interpolated from the interactive namespace
    before passing the line to the magic function.
    This is not always desirable, e.g. when the magic executes Python code
    (%timeit, %time, etc.).
    Decorate magics with `@no_var_expand` to opt-out of variable expansion.

    .. versionadded:: 7.3
    """
    ...

def output_can_be_silenced(magic_func):
    """Mark a magic function so its output may be silenced.

    The output is silenced if the Python code used as a parameter of
    the magic ends in a semicolon, not counting a Python comment that can
    follow it.
    """
    ...

line_magic = ...
cell_magic = ...
line_cell_magic = ...
register_line_magic = ...
register_cell_magic = ...
register_line_cell_magic = ...
class MagicsManager(Configurable):
    """Object that handles all magic-related functionality for IPython.
    """
    magics = ...
    lazy_magics = ...
    registry = ...
    shell = ...
    auto_magic = ...
    _auto_status = ...
    user_magics = ...
    def __init__(self, shell=..., config=..., user_magics=..., **traits) -> None:
        ...
    
    def auto_status(self):
        """Return descriptive string with automagic status."""
        ...
    
    def lsmagic(self): # -> Dict:
        """Return a dict of currently available magic functions.

        The return dict has the keys 'line' and 'cell', corresponding to the
        two types of magics we support.  Each value is a list of names.
        """
        ...
    
    def lsmagic_docs(self, brief=..., missing=...):
        """Return dict of documentation of magic functions.

        The return dict has the keys 'line' and 'cell', corresponding to the
        two types of magics we support. Each value is a dict keyed by magic
        name whose value is the function docstring. If a docstring is
        unavailable, the value of `missing` is used instead.

        If brief is True, only the first line of each docstring will be returned.
        """
        ...
    
    def register_lazy(self, name: str, fully_qualified_name: str): # -> None:
        """
        Lazily register a magic via an extension.


        Parameters
        ----------
        name : str
            Name of the magic you wish to register.
        fully_qualified_name :
            Fully qualified name of the module/submodule that should be loaded
            as an extensions when the magic is first called.
            It is assumed that loading this extensions will register the given
            magic.
        """
        ...
    
    def register(self, *magic_objects): # -> None:
        """Register one or more instances of Magics.

        Take one or more classes or instances of classes that subclass the main
        `core.Magic` class, and register them with IPython to use the magic
        functions they provide.  The registration process will then ensure that
        any methods that have decorated to provide line and/or cell magics will
        be recognized with the `%x`/`%%x` syntax as a line/cell magic
        respectively.

        If classes are given, they will be instantiated with the default
        constructor.  If your classes need a custom constructor, you should
        instanitate them first and pass the instance.

        The provided arguments can be an arbitrary mix of classes and instances.

        Parameters
        ----------
        *magic_objects : one or more classes or instances
        """
        ...
    
    def register_function(self, func, magic_kind=..., magic_name=...): # -> None:
        """Expose a standalone function as magic function for IPython.

        This will create an IPython magic (line, cell or both) from a
        standalone function.  The functions should have the following
        signatures:

        * For line magics: `def f(line)`
        * For cell magics: `def f(line, cell)`
        * For a function that does both: `def f(line, cell=None)`

        In the latter case, the function will be called with `cell==None` when
        invoked as `%f`, and with cell as a string when invoked as `%%f`.

        Parameters
        ----------
        func : callable
            Function to be registered as a magic.
        magic_kind : str
            Kind of magic, one of 'line', 'cell' or 'line_cell'
        magic_name : optional str
            If given, the name the magic will have in the IPython namespace.  By
            default, the name of the function itself is used.
        """
        ...
    
    def register_alias(self, alias_name, magic_name, magic_kind=..., magic_params=...): # -> None:
        """Register an alias to a magic function.

        The alias is an instance of :class:`MagicAlias`, which holds the
        name and kind of the magic it should call. Binding is done at
        call time, so if the underlying magic function is changed the alias
        will call the new function.

        Parameters
        ----------
        alias_name : str
            The name of the magic to be registered.
        magic_name : str
            The name of an existing magic.
        magic_kind : str
            Kind of magic, one of 'line' or 'cell'
        """
        ...
    


class Magics(Configurable):
    """Base class for implementing magic functions.

    Shell functions which can be reached as %function_name. All magic
    functions should accept a string, which they can parse for their own
    needs. This can make some functions easier to type, eg `%cd ../`
    vs. `%cd("../")`

    Classes providing magic functions need to subclass this class, and they
    MUST:

    - Use the method decorators `@line_magic` and `@cell_magic` to decorate
      individual methods as magic functions, AND

    - Use the class decorator `@magics_class` to ensure that the magic
      methods are properly registered at the instance level upon instance
      initialization.

    See :mod:`magic_functions` for examples of actual implementation classes.
    """
    options_table = ...
    magics = ...
    registered = ...
    shell = ...
    def __init__(self, shell=..., **kwargs) -> None:
        ...
    
    def arg_err(self, func): # -> None:
        """Print docstring if incorrect arguments were passed"""
        ...
    
    def format_latex(self, strng):
        """Format a string for latex inclusion."""
        ...
    
    def parse_options(self, arg_str, opt_str, *long_opts, **kw):
        """Parse options passed to an argument string.

        The interface is similar to that of :func:`getopt.getopt`, but it
        returns a :class:`~IPython.utils.struct.Struct` with the options as keys
        and the stripped argument string still as a string.

        arg_str is quoted as a true sys.argv vector by using shlex.split.
        This allows us to easily expand variables, glob files, quote
        arguments, etc.

        Parameters
        ----------
        arg_str : str
            The arguments to parse.
        opt_str : str
            The options specification.
        mode : str, default 'string'
            If given as 'list', the argument string is returned as a list (split
            on whitespace) instead of a string.
        list_all : bool, default False
            Put all option values in lists. Normally only options
            appearing more than once are put in a list.
        posix : bool, default True
            Whether to split the input line in POSIX mode or not, as per the
            conventions outlined in the :mod:`shlex` module from the standard
            library.
        """
        ...
    
    def default_option(self, fn, optstr): # -> None:
        """Make an entry in the options_table for fn, with value optstr"""
        ...
    


class MagicAlias:
    """An alias to another magic function.

    An alias is determined by its magic name and magic kind. Lookup
    is done at call time, so if the underlying magic changes the alias
    will call the new function.

    Use the :meth:`MagicsManager.register_alias` method or the
    `%alias_magic` magic function to create and register a new alias.
    """
    def __init__(self, shell, magic_name, magic_kind, magic_params=...) -> None:
        ...
    
    def __call__(self, *args, **kwargs):
        """Call the magic alias."""
        ...
    





"""
This type stub file was generated by pyright.
"""

"""Utilities for working with data structures like lists, dicts and tuples.
"""
def uniq_stable(elems):
    """uniq_stable(elems) -> list

    Return from an iterable, a list of all the unique elements in the input,
    but maintaining the order in which they first appear.

    Note: All elements in the input must be hashable for this routine
    to work, as it internally uses a set for efficiency reasons.
    """
    ...

def chop(seq, size):
    """Chop a sequence into chunks of the given size."""
    ...




from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    Optional,
    Sequence,
    Type,
    TypeVar,
    Union,
)

from django.forms.forms import BaseForm
from django.forms.models import BaseModelForm
from django.http import HttpRequest, HttpResponse
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
from django.views.generic.detail import (
    BaseDetailView,
    SingleObjectMixin,
    SingleObjectTemplateResponseMixin,
)
from typing_extensions import Literal

_FormT = TypeVar("_FormT", bound=BaseForm)

class AbstractFormMixin(ContextMixin):
    initial: Dict[str, Any] = ...
    form_class: Optional[Type[BaseForm]] = ...
    success_url: Optional[Union[str, Callable[..., Any]]] = ...
    prefix: Optional[str] = ...
    def get_initial(self) -> Dict[str, Any]: ...
    def get_prefix(self) -> Optional[str]: ...
    def get_form_kwargs(self) -> Dict[str, Any]: ...
    def get_success_url(self) -> str: ...

class FormMixin(Generic[_FormT], AbstractFormMixin):
    def get_form_class(self) -> Type[_FormT]: ...
    def get_form(self, form_class: Optional[Type[_FormT]] = ...) -> BaseForm: ...
    def form_valid(self, form: _FormT) -> HttpResponse: ...
    def form_invalid(self, form: _FormT) -> HttpResponse: ...

class ModelFormMixin(AbstractFormMixin, SingleObjectMixin):
    fields: Optional[Union[Sequence[str], Literal["__all__"]]] = ...
    def get_form_class(self) -> Type[BaseModelForm]: ...
    def get_form(
        self, form_class: Optional[Type[BaseModelForm]] = ...
    ) -> BaseModelForm: ...
    def form_valid(self, form: BaseModelForm) -> HttpResponse: ...
    def form_invalid(self, form: BaseModelForm) -> HttpResponse: ...

class ProcessFormView(View):
    def get(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def put(self, *args: str, **kwargs: Any) -> HttpResponse: ...

class BaseFormView(FormMixin[_FormT], ProcessFormView): ...
class FormView(TemplateResponseMixin, BaseFormView[_FormT]): ...
class BaseCreateView(ModelFormMixin, ProcessFormView): ...
class CreateView(SingleObjectTemplateResponseMixin, BaseCreateView): ...
class BaseUpdateView(ModelFormMixin, ProcessFormView): ...
class UpdateView(SingleObjectTemplateResponseMixin, BaseUpdateView): ...

class DeletionMixin:
    success_url: Optional[str] = ...
    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def delete(
        self, request: HttpRequest, *args: str, **kwargs: Any
    ) -> HttpResponse: ...
    def get_success_url(self) -> str: ...

class BaseDeleteView(DeletionMixin, BaseDetailView): ...
class DeleteView(SingleObjectTemplateResponseMixin, BaseDeleteView): ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class

"""Implementation of packaging-related magic functions.
"""
CONDA_COMMANDS_REQUIRING_PREFIX = ...
CONDA_COMMANDS_REQUIRING_YES = ...
CONDA_ENV_FLAGS = ...
CONDA_YES_FLAGS = ...
@magics_class
class PackagingMagics(Magics):
    """Magics related to packaging & installation"""
    @line_magic
    def pip(self, line): # -> None:
        """Run the pip package manager within the current kernel.

        Usage:
          %pip install [pkgs]
        """
        ...
    
    @line_magic
    def conda(self, line): # -> None:
        """Run the conda package manager within the current kernel.

        Usage:
          %conda install [pkgs]
        """
        ...
    





from typing import Mapping
from ...classes.graph import Graph

__all__ = ["min_cost_flow_cost", "min_cost_flow", "cost_of_flow", "max_flow_min_cost"]

def min_cost_flow_cost(
    G: Graph, demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
): ...
def min_cost_flow(
    G: Graph, demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
) -> Mapping: ...
def cost_of_flow(G: Graph, flowDict: Mapping, weight: str = "weight"): ...
def max_flow_min_cost(
    G: Graph, s, t, capacity: str = "capacity", weight: str = "weight"
) -> Mapping: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import logging
from weakref import WeakKeyDictionary

from ...gloo import Program
from ...gloo.preprocessor import preprocess
from ...util import logger
from ...util.event import EventEmitter
from .function import MainFunction
from .variable import Variable
from .compiler import Compiler

class ModularProgram(Program):
    def __init__(self, vcode="", fcode="", gcode=None): ...
    @property
    def vert(self): ...
    @vert.setter
    def vert(self, vcode): ...
    @property
    def frag(self): ...
    @frag.setter
    def frag(self, fcode): ...
    @property
    def geom(self): ...
    @geom.setter
    def geom(self, gcode): ...
    def _dep_changed(self, dep, code_changed=False, value_changed=False): ...
    def draw(self, *args, **kwargs): ...
    def build_if_needed(self): ...
    def _build(self): ...
    def update_variables(self): ...



from ._dict_learning import (
    dict_learning as dict_learning,
    dict_learning_online as dict_learning_online,
    sparse_encode as sparse_encode,
    DictionaryLearning as DictionaryLearning,
    MiniBatchDictionaryLearning as MiniBatchDictionaryLearning,
    SparseCoder as SparseCoder,
)
from ._fastica import FastICA as FastICA, fastica as fastica
from ..utils.extmath import randomized_svd as randomized_svd
from ._nmf import (
    NMF as NMF,
    MiniBatchNMF as MiniBatchNMF,
    non_negative_factorization as non_negative_factorization,
)
from ._kernel_pca import KernelPCA as KernelPCA
from ._sparse_pca import (
    SparsePCA as SparsePCA,
    MiniBatchSparsePCA as MiniBatchSparsePCA,
)
from ._incremental_pca import IncrementalPCA as IncrementalPCA
from ._lda import LatentDirichletAllocation as LatentDirichletAllocation
from ._factor_analysis import FactorAnalysis as FactorAnalysis
from ._pca import PCA as PCA
from ._truncated_svd import TruncatedSVD as TruncatedSVD


__all__ = [
    "DictionaryLearning",
    "FastICA",
    "IncrementalPCA",
    "KernelPCA",
    "MiniBatchDictionaryLearning",
    "MiniBatchNMF",
    "MiniBatchSparsePCA",
    "NMF",
    "PCA",
    "SparseCoder",
    "SparsePCA",
    "dict_learning",
    "dict_learning_online",
    "fastica",
    "non_negative_factorization",
    "randomized_svd",
    "sparse_encode",
    "FactorAnalysis",
    "TruncatedSVD",
    "LatentDirichletAllocation",
]



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from . import gl as gl  # noqa
from .wrappers import *  # noqa
from .context import (
    GLContext as GLContext,
    get_default_config as get_default_config,  # noqa
    get_current_canvas as get_current_canvas,
)  # noqa
from .globject import GLObject as GLObject  # noqa
from .buffer import VertexBuffer as VertexBuffer, IndexBuffer as IndexBuffer  # noqa
from .texture import (
    Texture1D as Texture1D,
    Texture2D as Texture2D,
    TextureAtlas as TextureAtlas,
    Texture3D as Texture3D,
    TextureCube as TextureCube,
    TextureEmulated3D as TextureEmulated3D,
)  # noqa
from .program import Program as Program  # noqa
from .framebuffer import (
    FrameBuffer as FrameBuffer,
    RenderBuffer as RenderBuffer,
)  # noqa
from . import util as util  # noqa



"""
This type stub file was generated by pyright.
"""

from typing import List

"""Implementations for various useful completers.

These are all loaded by default by IPython.
"""
_suffixes = ...
TIMEOUT_STORAGE = ...
TIMEOUT_GIVEUP = ...
import_re = ...
magic_run_re = ...
def module_list(path):
    """
    Return the list containing the names of the modules available in the given
    folder.
    """
    ...

def get_root_modules():
    """
    Returns a list containing the names of all the modules available in the
    folders of the pythonpath.

    ip.db['rootmodules_cache'] maps sys.path entries to list of modules.
    """
    ...

def is_importable(module, attr, only_modules):
    ...

def is_possible_submodule(module, attr):
    ...

def try_import(mod: str, only_modules=...) -> List[str]:
    """
    Try to import given module and return list of potential completions.
    """
    ...

def quick_completer(cmd, completions): # -> None:
    r""" Easily create a trivial completer for a command.

    Takes either a list of completions, or all completions in string (that will
    be split on whitespace).

    Example::

        [d:\ipython]|1> import ipy_completers
        [d:\ipython]|2> ipy_completers.quick_completer('foo', ['bar','baz'])
        [d:\ipython]|3> foo b<TAB>
        bar baz
        [d:\ipython]|3> foo ba
    """
    ...

def module_completion(line): # -> None:
    """
    Returns a list containing the completion possibilities for an import line.

    The line looks like this :
    'import xml.d'
    'from xml.dom import'
    """
    ...

def module_completer(self, event): # -> None:
    """Give completions after user has typed 'import ...' or 'from ...'"""
    ...

def magic_run_completer(self, event):
    """Complete files that end in .py or .ipy or .ipynb for the %run command.
    """
    ...

def cd_completer(self, event):
    """Completer function for cd, which only returns directories."""
    ...

def reset_completer(self, event):
    "A completer for %reset magic"
    ...




from typing import Callable
from ...classes.graph import Graph
from ...exception import NetworkXError

__all__ = [
    "could_be_isomorphic",
    "fast_could_be_isomorphic",
    "faster_could_be_isomorphic",
    "is_isomorphic",
]

def could_be_isomorphic(G1: Graph, G2: Graph): ...

graph_could_be_isomorphic = ...

def fast_could_be_isomorphic(G1: Graph, G2: Graph): ...

fast_graph_could_be_isomorphic = ...

def faster_could_be_isomorphic(G1: Graph, G2: Graph): ...

faster_graph_could_be_isomorphic = ...

def is_isomorphic(
    G1: Graph,
    G2: Graph,
    node_match: Callable | None = None,
    edge_match: Callable | None = None,
): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from os import path as _op

from .datasets import (
    load_iris as load_iris,
    load_crate as load_crate,
    load_data_file as load_data_file,  # noqa
    load_spatial_filters as load_spatial_filters,
)  # noqa
from .mesh import read_mesh as read_mesh, write_mesh as write_mesh  # noqa
from .image import (
    read_png as read_png,
    write_png as write_png,
    imread as imread,
    imsave as imsave,
    _make_png as _make_png,  # noqa
    _check_img_lib as _check_img_lib,
)  # noqa

_data_dir = ...

__all__ = [
    "imread",
    "imsave",
    "load_iris",
    "load_crate",
    "load_spatial_filters",
    "load_data_file",
    "read_mesh",
    "read_png",
    "write_mesh",
    "write_png",
]



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._highs._highs_constants, version: unspecified
import typing
import builtins as _mod_builtins

CONST_INF: float
CONST_I_INF: int
HIGHS_SIMPLEX_CRASH_STRATEGY_BIXBY: int
HIGHS_SIMPLEX_CRASH_STRATEGY_LTSF: int
HIGHS_SIMPLEX_CRASH_STRATEGY_OFF: int
HIGHS_SIMPLEX_DUAL_EDGE_WEIGHT_STRATEGY_DANTZIG: int
HIGHS_SIMPLEX_DUAL_EDGE_WEIGHT_STRATEGY_DEVEX: int
HIGHS_SIMPLEX_DUAL_EDGE_WEIGHT_STRATEGY_STEEPEST_EDGE: int
HIGHS_SIMPLEX_DUAL_EDGE_WEIGHT_STRATEGY_STEEPEST_EDGE_TO_DEVEX_SWITCH: int
HIGHS_SIMPLEX_PRIMAL_EDGE_WEIGHT_STRATEGY_DANTZIG: int
HIGHS_SIMPLEX_PRIMAL_EDGE_WEIGHT_STRATEGY_DEVEX: int
HIGHS_SIMPLEX_STRATEGY_CHOOSE: int
HIGHS_SIMPLEX_STRATEGY_DUAL: int
HIGHS_SIMPLEX_STRATEGY_PRIMAL: int
MESSAGE_LEVEL_ALWAYS: int
MESSAGE_LEVEL_DETAILED: int
MESSAGE_LEVEL_MINIMAL: int
MESSAGE_LEVEL_NONE: int
MESSAGE_LEVEL_VERBOSE: int
MODEL_STATUS_LOAD_ERROR: int
MODEL_STATUS_MODEL_EMPTY: int
MODEL_STATUS_MODEL_ERROR: int
MODEL_STATUS_NOTSET: int
MODEL_STATUS_OPTIMAL: int
MODEL_STATUS_POSTSOLVE_ERROR: int
MODEL_STATUS_PRESOLVE_ERROR: int
MODEL_STATUS_PRIMAL_INFEASIBLE: int
MODEL_STATUS_PRIMAL_UNBOUNDED: int
MODEL_STATUS_REACHED_DUAL_OBJECTIVE_VALUE_UPPER_BOUND: int
MODEL_STATUS_REACHED_ITERATION_LIMIT: int
MODEL_STATUS_REACHED_TIME_LIMIT: int
MODEL_STATUS_SOLVE_ERROR: int
__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def __getattr__(name) -> typing.Any:
    ...




from numpy import ndarray
from .._typing import Scalar

# -*- coding: utf-8 -*-

from math import pi
import numpy as np

def cubehelix(
    start: Scalar = 0.5,
    rot: Scalar = 1,
    gamma: Scalar = 1.0,
    reverse: bool = True,
    nlev: Scalar = 256.0,
    minSat: Scalar = 1.2,
    maxSat: Scalar = 1.2,
    minLight: Scalar = 0.0,
    maxLight: Scalar = 1.0,
    **kwargs,
) -> np.ndarray: ...



from .backend_agg import FigureCanvasAgg
from .backend_qt import FigureCanvasQT, _BackendQT

class FigureCanvasQTAgg(FigureCanvasAgg, FigureCanvasQT):
    def __init__(self, figure=...) -> None: ...
    def paintEvent(self, event)-> None: ...
    def print_figure(self, *args, **kwargs)-> None: ...

class _BackendQTAgg(_BackendQT):
    FigureCanvas = FigureCanvasQTAgg



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# vispy: testskip
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import sys
import os
import warnings
from os import path as op
from copy import deepcopy
from functools import partial

from ..util import use_log_level, run_subprocess
from ..util.ptime import time
from ._testing import has_application, nottest, IS_CI, IS_TRAVIS_CI

_line_sep = ...

class VispySkipSuite(Exception):
    def __init__(self, msg=""): ...

def _get_import_dir(): ...
def _unit(mode, extra_arg_string="", coverage=False): ...
def _docs(): ...
def _flake(): ...
def _check_line_endings(): ...

_script: str = ...

bad_examples: list = ...

def _skip_example(fname): ...
def _examples(fnames_str): ...
@nottest
def test(label: str = "full", extra_arg_string: str | ArrayLike = "", coverage: bool = False): ...



from skimage.future.graph.rag import RAG
from numpy.typing import NDArray, ArrayLike
import networkx as nx
import numpy as np
from . import _ncut

from scipy.sparse import linalg

def cut_threshold(
    labels: NDArray, rag: RAG, thresh: float, in_place: bool = True
) -> NDArray: ...
def cut_normalized(
    labels: NDArray,
    rag: RAG,
    thresh: float = 0.001,
    num_cuts: int = 10,
    in_place: bool = True,
    max_edge: float = 1.0,
    *,
    random_state=None,
) -> NDArray: ...
def partition_by_cut(cut: ArrayLike, rag: RAG) -> RAG: ...
def get_min_ncut(
    ev: ArrayLike, d: NDArray, w: NDArray, num_cuts: int
) -> tuple[ArrayLike, float]: ...
def _label_all(rag, attr_name): ...
def _ncut_relabel(rag, thresh, num_cuts, random_state): ...



from .collections import PolyCollection
from typing import Literal, Sequence
from ._typing import *

__all__ = ["stackplot"]

def stackplot(
    axes,
    x: ArrayLike,
    *args,
    labels: Sequence[str] = ...,
    colors: Sequence[Color] = ...,
    baseline: Literal["zero", "sym", "wiggle", "weighted_wiggle"] = ...,
    **kwargs
) -> list[PolyCollection]: ...



from collections.abc import Iterable
from ...classes.graph import Graph

from collections import Counter

from ...algorithms.components import is_connected
from ...exception import NetworkXError
from ...utils import groups, not_implemented_for, py_random_state

__all__ = ["asyn_fluidc"]

@py_random_state(3)
def asyn_fluidc(G: Graph, k, max_iter=100, seed=None) -> Iterable: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.special.specfun, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def airyzo(nt, kf=...) -> typing.Any:
    "xa,xb,xc,xd = airyzo(nt,[kf])\n\nWrapper for ``airyzo``.\n\nParameters\n----------\nnt : input int\n\nOther Parameters\n----------------\nkf : input int, optional\n    Default: 1\n\nReturns\n-------\nxa : rank-1 array('d') with bounds (nt)\nxb : rank-1 array('d') with bounds (nt)\nxc : rank-1 array('d') with bounds (nt)\nxd : rank-1 array('d') with bounds (nt)\n"
    ...

def bernob(n) -> typing.Any:
    "bn = bernob(n)\n\nWrapper for ``bernob``.\n\nParameters\n----------\nn : input int\n\nReturns\n-------\nbn : rank-1 array('d') with bounds (n + 1)\n"
    ...

def cerzo(nt) -> typing.Any:
    "zo = cerzo(nt)\n\nWrapper for ``cerzo``.\n\nParameters\n----------\nnt : input int\n\nReturns\n-------\nzo : rank-1 array('D') with bounds (nt)\n"
    ...

def clpmn(m, n, x, y, ntype) -> typing.Any:
    "cpm,cpd = clpmn(m,n,x,y,ntype)\n\nWrapper for ``clpmn``.\n\nParameters\n----------\nm : input int\nn : input int\nx : input float\ny : input float\nntype : input int\n\nReturns\n-------\ncpm : rank-2 array('D') with bounds (m + 1,n + 1)\ncpd : rank-2 array('D') with bounds (m + 1,n + 1)\n"
    ...

def clpn(n, z) -> typing.Any:
    "cpn,cpd = clpn(n,z)\n\nWrapper for ``clpn``.\n\nParameters\n----------\nn : input int\nz : input complex\n\nReturns\n-------\ncpn : rank-1 array('D') with bounds (n + 1)\ncpd : rank-1 array('D') with bounds (n + 1)\n"
    ...

def clqmn(m, n, z) -> typing.Any:
    "cqm,cqd = clqmn(m,n,z)\n\nWrapper for ``clqmn``.\n\nParameters\n----------\nm : input int\nn : input int\nz : input complex\n\nReturns\n-------\ncqm : rank-2 array('D') with bounds (mm + 1,n + 1)\ncqd : rank-2 array('D') with bounds (mm + 1,n + 1)\n"
    ...

def clqn(n, z) -> typing.Any:
    "cqn,cqd = clqn(n,z)\n\nWrapper for ``clqn``.\n\nParameters\n----------\nn : input int\nz : input complex\n\nReturns\n-------\ncqn : rank-1 array('D') with bounds (n + 1)\ncqd : rank-1 array('D') with bounds (n + 1)\n"
    ...

def cpbdn(n, z) -> typing.Any:
    "cpb,cpd = cpbdn(n,z)\n\nWrapper for ``cpbdn``.\n\nParameters\n----------\nn : input int\nz : input complex\n\nReturns\n-------\ncpb : rank-1 array('D') with bounds (abs(n)+2)\ncpd : rank-1 array('D') with bounds (abs(n)+2)\n"
    ...

def cyzo(nt, kf, kc) -> typing.Any:
    "zo,zv = cyzo(nt,kf,kc)\n\nWrapper for ``cyzo``.\n\nParameters\n----------\nnt : input int\nkf : input int\nkc : input int\n\nReturns\n-------\nzo : rank-1 array('D') with bounds (nt)\nzv : rank-1 array('D') with bounds (nt)\n"
    ...

def eulerb(n) -> typing.Any:
    "en = eulerb(n)\n\nWrapper for ``eulerb``.\n\nParameters\n----------\nn : input int\n\nReturns\n-------\nen : rank-1 array('d') with bounds (n + 1)\n"
    ...

def fcoef(kd, m, q, a) -> typing.Any:
    "fc = fcoef(kd,m,q,a)\n\nWrapper for ``fcoef``.\n\nParameters\n----------\nkd : input int\nm : input int\nq : input float\na : input float\n\nReturns\n-------\nfc : rank-1 array('d') with bounds (251)\n"
    ...

def fcszo(kf, nt) -> typing.Any:
    "zo = fcszo(kf,nt)\n\nWrapper for ``fcszo``.\n\nParameters\n----------\nkf : input int\nnt : input int\n\nReturns\n-------\nzo : rank-1 array('D') with bounds (nt)\n"
    ...

def jdzo(nt) -> typing.Any:
    "n,m,pcode,zo = jdzo(nt)\n\nWrapper for ``jdzo``.\n\nParameters\n----------\nnt : input int\n\nReturns\n-------\nn : rank-1 array('i') with bounds (1400)\nm : rank-1 array('i') with bounds (1400)\npcode : rank-1 array('i') with bounds (1400)\nzo : rank-1 array('d') with bounds (1401)\n"
    ...

def jyzo(n, nt) -> typing.Any:
    "rj0,rj1,ry0,ry1 = jyzo(n,nt)\n\nWrapper for ``jyzo``.\n\nParameters\n----------\nn : input int\nnt : input int\n\nReturns\n-------\nrj0 : rank-1 array('d') with bounds (nt)\nrj1 : rank-1 array('d') with bounds (nt)\nry0 : rank-1 array('d') with bounds (nt)\nry1 : rank-1 array('d') with bounds (nt)\n"
    ...

def klvnzo(nt, kd) -> typing.Any:
    "zo = klvnzo(nt,kd)\n\nWrapper for ``klvnzo``.\n\nParameters\n----------\nnt : input int\nkd : input int\n\nReturns\n-------\nzo : rank-1 array('d') with bounds (nt)\n"
    ...

def lamn(n, x) -> typing.Any:
    "nm,bl,dl = lamn(n,x)\n\nWrapper for ``lamn``.\n\nParameters\n----------\nn : input int\nx : input float\n\nReturns\n-------\nnm : int\nbl : rank-1 array('d') with bounds (n + 1)\ndl : rank-1 array('d') with bounds (n + 1)\n"
    ...

def lamv(v, x) -> typing.Any:
    "vm,vl,dl = lamv(v,x)\n\nWrapper for ``lamv``.\n\nParameters\n----------\nv : input float\nx : input float\n\nReturns\n-------\nvm : float\nvl : rank-1 array('d') with bounds ((int)v+1)\ndl : rank-1 array('d') with bounds ((int)v+1)\n"
    ...

def lpmn(m, n, x) -> typing.Any:
    "pm,pd = lpmn(m,n,x)\n\nWrapper for ``lpmn``.\n\nParameters\n----------\nm : input int\nn : input int\nx : input float\n\nReturns\n-------\npm : rank-2 array('d') with bounds (m + 1,n + 1)\npd : rank-2 array('d') with bounds (m + 1,n + 1)\n"
    ...

def lpn(n, x) -> typing.Any:
    "pn,pd = lpn(n,x)\n\nWrapper for ``lpn``.\n\nParameters\n----------\nn : input int\nx : input float\n\nReturns\n-------\npn : rank-1 array('d') with bounds (n + 1)\npd : rank-1 array('d') with bounds (n + 1)\n"
    ...

def lqmn(m, n, x) -> typing.Any:
    "qm,qd = lqmn(m,n,x)\n\nWrapper for ``lqmn``.\n\nParameters\n----------\nm : input int\nn : input int\nx : input float\n\nReturns\n-------\nqm : rank-2 array('d') with bounds (mm + 1,n + 1)\nqd : rank-2 array('d') with bounds (mm + 1,n + 1)\n"
    ...

def lqnb(n, x) -> typing.Any:
    "qn,qd = lqnb(n,x)\n\nWrapper for ``lqnb``.\n\nParameters\n----------\nn : input int\nx : input float\n\nReturns\n-------\nqn : rank-1 array('d') with bounds (n + 1)\nqd : rank-1 array('d') with bounds (n + 1)\n"
    ...

def pbdv(v, x) -> typing.Any:
    "dv,dp,pdf,pdd = pbdv(v,x)\n\nWrapper for ``pbdv``.\n\nParameters\n----------\nv : input float\nx : input float\n\nReturns\n-------\ndv : rank-1 array('d') with bounds (abs((int)v)+2)\ndp : rank-1 array('d') with bounds (abs((int)v)+2)\npdf : float\npdd : float\n"
    ...

def rctj(n, x) -> typing.Any:
    "nm,rj,dj = rctj(n,x)\n\nWrapper for ``rctj``.\n\nParameters\n----------\nn : input int\nx : input float\n\nReturns\n-------\nnm : int\nrj : rank-1 array('d') with bounds (n + 1)\ndj : rank-1 array('d') with bounds (n + 1)\n"
    ...

def rcty(n, x) -> typing.Any:
    "nm,ry,dy = rcty(n,x)\n\nWrapper for ``rcty``.\n\nParameters\n----------\nn : input int\nx : input float\n\nReturns\n-------\nnm : int\nry : rank-1 array('d') with bounds (n + 1)\ndy : rank-1 array('d') with bounds (n + 1)\n"
    ...

def segv(m, n, c, kd) -> typing.Any:
    "cv,eg = segv(m,n,c,kd)\n\nWrapper for ``segv``.\n\nParameters\n----------\nm : input int\nn : input int\nc : input float\nkd : input int\n\nReturns\n-------\ncv : float\neg : rank-1 array('d') with bounds (n-m+2)\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .base_filter import Filter
from ..transforms import NullTransform
from ...geometry import Rect

class Clipper(Filter):

    FRAG_SHADER: str = ...

    def __init__(self, bounds=..., transform=None): ...
    @property
    def bounds(self): ...
    @bounds.setter
    def bounds(self, b): ...
    @property
    def transform(self): ...
    @transform.setter
    def transform(self, tr): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.spatial._distance_wrap, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def cdist_braycurtis_double_wrap() -> typing.Any:
    ...

def cdist_canberra_double_wrap() -> typing.Any:
    ...

def cdist_chebyshev_double_wrap() -> typing.Any:
    ...

def cdist_cityblock_double_wrap() -> typing.Any:
    ...

def cdist_correlation_double_wrap(XA, XB, dm, **kwargs) -> typing.Any:
    ...

def cdist_cosine_double_wrap() -> typing.Any:
    ...

def cdist_dice_bool_wrap() -> typing.Any:
    ...

def cdist_euclidean_double_wrap() -> typing.Any:
    ...

def cdist_hamming_bool_wrap() -> typing.Any:
    ...

def cdist_hamming_double_wrap() -> typing.Any:
    ...

def cdist_jaccard_bool_wrap() -> typing.Any:
    ...

def cdist_jaccard_double_wrap() -> typing.Any:
    ...

def cdist_jensenshannon_double_wrap() -> typing.Any:
    ...

def cdist_kulsinski_bool_wrap() -> typing.Any:
    ...

def cdist_mahalanobis_double_wrap() -> typing.Any:
    ...

def cdist_minkowski_double_wrap() -> typing.Any:
    ...

def cdist_old_weighted_minkowski_double_wrap() -> typing.Any:
    ...

def cdist_rogerstanimoto_bool_wrap() -> typing.Any:
    ...

def cdist_russellrao_bool_wrap() -> typing.Any:
    ...

def cdist_seuclidean_double_wrap() -> typing.Any:
    ...

def cdist_sokalmichener_bool_wrap() -> typing.Any:
    ...

def cdist_sokalsneath_bool_wrap() -> typing.Any:
    ...

def cdist_sqeuclidean_double_wrap() -> typing.Any:
    ...

def cdist_weighted_chebyshev_double_wrap() -> typing.Any:
    ...

def cdist_weighted_minkowski_double_wrap() -> typing.Any:
    ...

def cdist_yule_bool_wrap() -> typing.Any:
    ...

def pdist_braycurtis_double_wrap() -> typing.Any:
    ...

def pdist_canberra_double_wrap() -> typing.Any:
    ...

def pdist_chebyshev_double_wrap() -> typing.Any:
    ...

def pdist_cityblock_double_wrap() -> typing.Any:
    ...

def pdist_correlation_double_wrap(X, dm, **kwargs) -> typing.Any:
    ...

def pdist_cosine_double_wrap() -> typing.Any:
    ...

def pdist_dice_bool_wrap() -> typing.Any:
    ...

def pdist_euclidean_double_wrap() -> typing.Any:
    ...

def pdist_hamming_bool_wrap() -> typing.Any:
    ...

def pdist_hamming_double_wrap() -> typing.Any:
    ...

def pdist_jaccard_bool_wrap() -> typing.Any:
    ...

def pdist_jaccard_double_wrap() -> typing.Any:
    ...

def pdist_jensenshannon_double_wrap() -> typing.Any:
    ...

def pdist_kulsinski_bool_wrap() -> typing.Any:
    ...

def pdist_mahalanobis_double_wrap() -> typing.Any:
    ...

def pdist_minkowski_double_wrap() -> typing.Any:
    ...

def pdist_old_weighted_minkowski_double_wrap() -> typing.Any:
    ...

def pdist_rogerstanimoto_bool_wrap() -> typing.Any:
    ...

def pdist_russellrao_bool_wrap() -> typing.Any:
    ...

def pdist_seuclidean_double_wrap() -> typing.Any:
    ...

def pdist_sokalmichener_bool_wrap() -> typing.Any:
    ...

def pdist_sokalsneath_bool_wrap() -> typing.Any:
    ...

def pdist_sqeuclidean_double_wrap() -> typing.Any:
    ...

def pdist_weighted_chebyshev_double_wrap() -> typing.Any:
    ...

def pdist_weighted_minkowski_double_wrap() -> typing.Any:
    ...

def pdist_yule_bool_wrap() -> typing.Any:
    ...

def to_squareform_from_vector_wrap() -> typing.Any:
    ...

def to_vector_from_squareform_wrap() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.
"""

import resource

"""
Utilities for timing code execution.
"""
if resource is not None and hasattr(resource, "getrusage"):
    def clocku():
        """clocku() -> floating point number

        Return the *USER* CPU time in seconds since the start of the process.
        This is done via a call to resource.getrusage, so it avoids the
        wraparound problems in time.clock()."""
        ...
    
    def clocks():
        """clocks() -> floating point number

        Return the *SYSTEM* CPU time in seconds since the start of the process.
        This is done via a call to resource.getrusage, so it avoids the
        wraparound problems in time.clock()."""
        ...
    
    def clock():
        """clock() -> floating point number

        Return the *TOTAL USER+SYSTEM* CPU time in seconds since the start of
        the process.  This is done via a call to resource.getrusage, so it
        avoids the wraparound problems in time.clock()."""
        ...
    
    def clock2():
        """clock2() -> (t_user,t_system)

        Similar to clock(), but return a tuple of user/system times."""
        ...
    
else:
    clock = ...
    def clock2():
        """Under windows, system CPU time can't be measured.

        This just returns process_time() and zero."""
        ...
    
def timings_out(reps, func, *args, **kw):
    """timings_out(reps,func,*args,**kw) -> (t_total,t_per_call,output)

    Execute a function reps times, return a tuple with the elapsed total
    CPU time in seconds, the time per call and the function's output.

    Under Unix, the return value is the sum of user+system time consumed by
    the process, computed via the resource module.  This prevents problems
    related to the wraparound effect which the time.clock() function has.

    Under Windows the return value is in wall clock seconds. See the
    documentation for the time module for more details."""
    ...

def timings(reps, func, *args, **kw):
    """timings(reps,func,*args,**kw) -> (t_total,t_per_call)

    Execute a function reps times, return a tuple with the elapsed total CPU
    time in seconds and the time per call. These are just the first two values
    in timings_out()."""
    ...

def timing(func, *args, **kw):
    """timing(func,*args,**kw) -> t_total

    Execute a function once, return the elapsed total CPU time in
    seconds. This is just the first value in timings_out()."""
    ...




from typing import Mapping
from ...classes.graph import Graph
from ...utils import not_implemented_for

# Authors: Erwan Le Merrer (erwan.lemerrer@technicolor.com)

__all__ = ["second_order_centrality"]

def second_order_centrality(G: Graph) -> Mapping: ...



from typing import Mapping

from ...classes.graph import Graph

__all__ = ["percolation_centrality"]

def percolation_centrality(
    G: Graph, attribute="percolation", states=None, weight=None
) -> Mapping: ...



from collections import deque

from ..classes.graph import Graph
from ..utils import UnionFind, not_implemented_for

__all__ = ["d_separated"]

def d_separated(G: Graph, x: set, y: set, z: set) -> bool: ...



from typing import (
    Any,
    Callable,
    Dict,
    Generic,
    Iterable,
    Optional,
    Tuple,
    TypeVar,
    Union,
    overload,
)

from django.db.models import Field, Transform
from typing_extensions import Literal

from .mixins import CheckFieldDefaultMixin

_Choice = Tuple[Any, Any]
_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]
_ValidatorCallable = Callable[..., None]
_ErrorMessagesToOverride = Dict[str, Any]

_T = TypeVar("_T", bound="Optional[Dict[str, Optional[str]]]")

class HStoreField(Generic[_T], CheckFieldDefaultMixin, Field[Any, Any]):
    @overload
    def __init__(
        self: HStoreField[Dict[str, Optional[str]]],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[False] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self: HStoreField[Optional[Dict[str, Optional[str]]]],
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: Literal[True] = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    def get_transform(self, name: Any) -> Any: ...
    def __get__(self: HStoreField[_T], instance: Any, owner: Any) -> _T: ...
    def __set__(self: HStoreField[_T], instance: Any, value: _T) -> None: ...

class KeyTransform(Transform):
    def __init__(self, key_name: str, *args: Any, **kwargs: Any) -> None: ...

class KeyTransformFactory:
    def __init__(self, key_name: str) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> KeyTransform: ...

class KeysTransform(Transform): ...
class ValuesTransform(Transform): ...



# This list of English stop words is taken from the "Glasgow Information
# Retrieval Group". The original list can be found at
# http://ir.dcs.gla.ac.uk/resources/linguistic_utils/stop_words
ENGLISH_STOP_WORDS = ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from operator import itemgetter as itemgetter
from .kernels import Kernel
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.linalg import (
    cholesky as cholesky,
    cho_solve as cho_solve,
    solve_triangular as solve_triangular,
)
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, RegressorMixin, clone as clone, MultiOutputMixin
from .kernels import RBF as RBF, ConstantKernel as C
from .._typing import ArrayLike, Int, MatrixLike, Float
from ..utils import check_random_state as check_random_state

GaussianProcessRegressor_Self = TypeVar(
    "GaussianProcessRegressor_Self", bound="GaussianProcessRegressor"
)


# Authors: Jan Hendrik Metzen <jhm@informatik.uni-bremen.de>
# Modified by: Pete Green <p.l.green@liverpool.ac.uk>
# License: BSD 3 clause

import warnings

import numpy as np
import scipy.optimize

GPR_CHOLESKY_LOWER: bool = ...


class GaussianProcessRegressor(MultiOutputMixin, RegressorMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    log_marginal_likelihood_value_: float = ...
    alpha_: ArrayLike = ...
    L_: ArrayLike = ...
    kernel_: Kernel = ...
    y_train_: ArrayLike = ...
    X_train_: ArrayLike | list[Any] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: None | Kernel = None,
        *,
        alpha: float | ArrayLike = 1e-10,
        optimizer: Literal["fmin_l_bfgs_b", "fmin_l_bfgs_b"]
        | None
        | Callable = "fmin_l_bfgs_b",
        n_restarts_optimizer: Int = 0,
        normalize_y: bool = False,
        copy_X_train: bool = True,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: GaussianProcessRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
    ) -> GaussianProcessRegressor_Self:
        ...

    def predict(
        self,
        X: MatrixLike | ArrayLike,
        return_std: bool = False,
        return_cov: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray, ndarray] | tuple[ndarray, ndarray]:
        ...

    def sample_y(
        self,
        X: MatrixLike | ArrayLike,
        n_samples: Int = 1,
        random_state: RandomState | None | Int = 0,
    ) -> ndarray:
        ...

    def log_marginal_likelihood(
        self,
        theta: None | ArrayLike = None,
        eval_gradient: bool = False,
        clone_kernel: bool = True,
    ) -> tuple[float, ndarray] | tuple[Float, ndarray] | Float:
        ...



from np import dtype
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------
import numpy as np

class ArrayList(object):
    def __init__(
        self,
        data: ArrayLike | None = None,
        itemsize: int | ArrayLike | None = None,
        dtype: np.dtype = ...,
        sizeable: bool = True,
        writeable: bool = True,
    ): ...
    @property
    def data(self): ...
    @property
    def size(self): ...
    @property
    def itemsize(self): ...
    @property
    def dtype(self): ...
    def reserve(self, capacity): ...
    def __len__(self): ...
    def __str__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, data): ...
    def __delitem__(self, key): ...
    def insert(self, index: int, data: ArrayLike, itemsize: int | ArrayLike | None = None): ...
    def append(self, data: ArrayLike, itemsize: int | ArrayLike | None = None): ...



from typing import Mapping
from operator import itemgetter

from ...classes.graph import Graph

__all__ = ["load_centrality", "edge_load_centrality"]

def newman_betweenness_centrality(
    G: Graph, v=None, cutoff=None, normalized=True, weight=None
) -> Mapping: ...

load_centrality = ...

def edge_load_centrality(G: Graph, cutoff=False): ...



from matplotlib.axis import XAxis, YAxis
from matplotlib.cbook import Grouper
from matplotlib.image import AxesImage
from matplotlib.legend import Legend
from matplotlib.table import Table
import numpy as np
from typing import Any, Callable, Collection, Literal, MutableSequence, Sequence, overload
from matplotlib._typing import *
from matplotlib.transforms import Bbox, BboxBase, Transform, TransformedBbox
from matplotlib.backend_bases import RendererBase
from matplotlib.patches import Patch
from matplotlib.backend_bases import MouseButton, MouseEvent
from matplotlib.container import Container
from matplotlib.lines import Line2D
from matplotlib.figure import Figure
from matplotlib.scale import ScaleBase
from matplotlib.artist import Artist, allow_rasterization
from ._axes import Axes

class _axis_method_wrapper:
    def __init__(self, attr_name, method_name, *, doc_sub=...) -> None: ...
    def __set_name__(self, owner, name) -> None: ...

class _AxesBase(Artist):
    name: str = ...

    def __str__(self) -> str: ...
    def __init__(
        self,
        fig: Figure,
        rect: Sequence[float],
        *,
        facecolor: Color|None=None,
        frameon: bool = True,
        sharex: Axes|None = None,
        sharey: Axes|None = None,
        label: str='',
        xscale: float|None=None,
        yscale: float|None=None,
        box_aspect: float|None = None,
        **kwargs
    ) -> None: ...
    def __getstate__(self) -> dict[str, Any]: ...
    def __setstate__(self, state)-> None: ...
    def __repr__(self)-> str: ...
    def get_window_extent(self, renderer=..., *args, **kwargs) -> TransformedBbox: ...
    def set_figure(self, fig: Figure)-> None: ...
    @property
    def viewLim(self)-> Bbox: ...
    def get_xaxis_transform(self, which=...) -> Transform: ...
    def get_xaxis_text1_transform(self, pad_points) -> tuple[Transform, str, str]: ...
    def get_xaxis_text2_transform(self, pad_points) -> tuple[Transform, str, str]: ...
    def get_yaxis_transform(self, which=...) -> Transform: ...
    def get_yaxis_text1_transform(self, pad_points) -> tuple[Transform, str, str]: ...
    def get_yaxis_text2_transform(self, pad_points) -> tuple[Transform, str, str]: ...
    def get_position(self, original: bool = ...) -> Bbox: ...
    def set_position(
        self,
        pos: Sequence[float] | Bbox,
        which: Literal["both", "active", "original"] = ...,
    ) -> None: ...
    def reset_position(self) -> None: ...
    def set_axes_locator(
        self, locator: Callable[[Axes, RendererBase], Bbox]
    ) -> None: ...
    def get_axes_locator(self) -> Callable[[Axes, RendererBase], Bbox]: ...
    def sharex(self, other: Axes) -> None: ...
    def sharey(self, other: Axes) -> None: ...
    def clear(self) -> None: ...
    @property
    def artists(self) -> MutableSequence: ...
    @property
    def collections(self): ...
    @property
    def images(self)-> MutableSequence: ...
    @property
    def lines(self)-> MutableSequence: ...
    @property
    def patches(self)-> MutableSequence: ...
    @property
    def tables(self)-> MutableSequence: ...
    @property
    def texts(self)-> MutableSequence: ...
    def cla(self) -> None: ...
    def get_facecolor(self) -> Color: ...
    def set_facecolor(self, color: Color)-> None: ...
    def set_prop_cycle(self, *args, **kwargs)-> None: ...
    def get_aspect(self) -> Literal["auto"] | float: ...
    def set_aspect(
        self,
        aspect: Literal["auto", "equal"] | float,
        adjustable: None | Literal["box", "datalim"] = ...,
        anchor: None | str | Sequence[float] = ...,
        share: bool = False,
    )-> None: ...
    def get_adjustable(self) -> str: ...
    def set_adjustable(
        self, adjustable: Literal["box", "datalim"], share: bool = False
    )-> None: ...
    def get_box_aspect(self) -> float | None: ...
    def set_box_aspect(self, aspect: float | None = ...) -> None: ...
    def get_anchor(self) -> str: ...
    def set_anchor(
        self,
        anchor: Literal["C", "SW", "S", "SE", "E", "NE", "N", "NW", "W"],
        share: bool = False,
    )-> None: ...
    def get_data_ratio(self)-> float: ...
    def apply_aspect(self, position=...)-> None: ...
    def axis(self, *args, emit: bool = ..., **kwargs)-> tuple[float, float, float, float]: ...
    def get_legend(self) -> Legend | None: ...
    def get_images(self) -> list[AxesImage]: ...
    def get_lines(self) -> list[Line2D]: ...
    def get_xaxis(self) -> XAxis: ...
    def get_yaxis(self) -> YAxis: ...
    get_xgridlines = ...
    get_xticklines = ...
    get_ygridlines = ...
    get_yticklines = ...
    def has_data(self) -> bool: ...
    def add_artist(self, a: Artist) -> Artist: ...
    def add_child_axes(self, ax: _AxesBase) -> Axes: ...
    def add_collection(self, collection: Collection, autolim=...) -> Collection: ...
    def add_image(self, image: AxesImage) -> AxesImage: ...
    def add_line(self, line: Line2D) -> Line2D: ...
    def add_patch(self, p: Patch) -> Patch: ...
    def add_table(self, tab: Table) -> Table: ...
    def add_container(self, container: Container) -> Container: ...
    def relim(self, visible_only: bool = ...)-> None: ...
    def update_datalim(self, xys, updatex: bool = ..., updatey: bool = ...)-> None: ...
    def in_axes(self, mouseevent) -> bool: ...
    get_autoscalex_on = ...
    get_autoscaley_on = ...
    set_autoscalex_on = ...
    set_autoscaley_on = ...
    def get_autoscale_on(self) -> bool: ...
    def set_autoscale_on(self, b: bool)-> None: ...
    @property
    def use_sticky_edges(self) -> bool: ...
    @use_sticky_edges.setter
    def use_sticky_edges(self, b: bool)-> None: ...
    def set_xmargin(self, m: float)-> None: ...
    def set_ymargin(self, m: float)-> None: ...
    def margins(
        self, *margins, x: float = ..., y: float = ..., tight: bool | None = True
    ) -> tuple[float, float]: ...
    def set_rasterization_zorder(self, z: float | None) -> None: ...
    def get_rasterization_zorder(self) -> float: ...
    def autoscale(
        self,
        enable: bool | None = ...,
        axis: Literal["both", "x", "y"] = ...,
        tight: bool | None = ...,
    ) -> None: ...
    def autoscale_view(
        self, tight: bool | None = ..., scalex: bool = True, scaley: bool = True
    ) -> None: ...
    @allow_rasterization
    def draw(self, renderer)-> None: ...
    def draw_artist(self, a: Artist) -> None: ...
    def redraw_in_frame(self) -> None: ...
    def get_renderer_cache(self): ...
    def get_frame_on(self) -> bool: ...
    def set_frame_on(self, b: bool) -> None: ...
    def get_axisbelow(self) -> tuple[bool, Literal["line"]]: ...
    def set_axisbelow(self, b: bool | Literal["line"]) -> None: ...
    def grid(
        self,
        visible: bool | None = ...,
        which: Literal["major", "minor", "both"] = ...,
        axis: Literal["both", "x", "y"] = ...,
        **kwargs
    ) -> None: ...
    def ticklabel_format(
        self,
        *,
        axis: Literal["x", "y", "both"] = ...,
        style: Literal["sci", "scientific", "plain"] = ...,
        scilimits=...,
        useOffset: bool | float = ...,
        useLocale: bool = ...,
        useMathText: bool = ...
    )-> None: ...
    def locator_params(
        self, axis: Literal["both", "x", "y"] = ..., tight: bool | None = ..., **kwargs
    ) -> None: ...
    def tick_params(self, axis: Literal["x", "y", "both"] = ..., **kwargs) -> None: ...
    def set_axis_off(self) -> None: ...
    def set_axis_on(self) -> None: ...
    def get_xlabel(self) -> str: ...
    def set_xlabel(
        self,
        xlabel: str,
        fontdict=...,
        labelpad: float = ...,
        *,
        loc: Literal["left", "center", "right"] = ...,
        **kwargs
    )-> None: ...
    def invert_xaxis(self) -> None: ...
    xaxis_inverted = ...
    def get_xbound(self) -> tuple[float, float]: ...
    def set_xbound(
        self, lower: float | None = ..., upper: float | None = ...
    ) -> None: ...
    def get_xlim(self) -> tuple[float, float]: ...
    @overload
    def set_xlim(
        self,
        left: tuple[float | np.datetime64, float | np.datetime64],
        *,
        emit: bool = ...,
        auto: bool | None = ...,
        xmin: float = ...,
        xmax: float = ...
    ) -> tuple[float, float]: ...
    @overload
    def set_xlim(
        self,
        left: float | np.datetime64 = ...,
        right: float | np.datetime64 = ...,
        emit: bool = ...,
        auto: bool | None = ...,
        *,
        xmin: float = ...,
        xmax: float = ...
    ) -> tuple[float, float]: ...
    def get_xscale(self) -> str: ...
    def set_xscale(
        self, value: Literal["linear", "log", "symlog", "logit"] | ScaleBase, **kwargs
    )-> None: ...
    get_xticks = ...
    set_xticks = ...
    get_xmajorticklabels = ...
    get_xminorticklabels = ...
    get_xticklabels = ...
    set_xticklabels = ...
    def get_ylabel(self) -> str: ...
    def set_ylabel(
        self,
        ylabel: str,
        fontdict=...,
        labelpad: float = ...,
        *,
        loc: Literal["bottom", "center", "top"] = ...,
        **kwargs
    ) -> None: ...
    def invert_yaxis(self) -> None: ...
    yaxis_inverted = ...
    def get_ybound(self) -> tuple[float, float]: ...
    def set_ybound(
        self, lower: float | None = ..., upper: float | None = ...
    ) -> None: ...
    def get_ylim(self) -> tuple[float, float]: ...
    def set_ylim(
        self,
        bottom: float = ...,
        top: float = ...,
        emit: bool = ...,
        auto: bool | None = ...,
        *,
        ymin: float = ...,
        ymax: float = ...
    )-> None: ...
    get_yscale = ...
    def set_yscale(
        self, value: Literal["linear", "log", "symlog", "logit"] | ScaleBase, **kwargs
    ) -> None: ...
    get_yticks = ...
    set_yticks = ...
    get_ymajorticklabels = ...
    get_yminorticklabels = ...
    get_yticklabels = ...
    set_yticklabels = ...
    xaxis_date = ...
    yaxis_date = ...
    def format_xdata(self, x) -> str: ...
    def format_ydata(self, y) -> str: ...
    def format_coord(self, x, y) -> str: ...
    def minorticks_on(self) -> None: ...
    def minorticks_off(self) -> None: ...
    def can_zoom(self) -> bool: ...
    def can_pan(self) -> bool: ...
    def get_navigate(self) -> bool: ...
    def set_navigate(self, b: bool) -> None: ...
    def get_navigate_mode(self) -> str | None: ...
    def set_navigate_mode(self, b: str | None) -> None: ...
    def start_pan(self, x: float, y: float, button: MouseButton) -> None: ...
    def end_pan(self) -> None: ...
    def drag_pan(
        self, button: MouseButton, key: str | None, x: float, y: float
    ) -> None: ...
    def get_children(self)-> list: ...
    def contains(self, mouseevent: MouseEvent) -> bool: ...
    def contains_point(self, point) -> bool: ...
    def get_default_bbox_extra_artists(self) -> list[Artist]: ...
    def get_tightbbox(
        self,
        renderer: RendererBase = ...,
        call_axes_locator: bool = ...,
        bbox_extra_artists: list | None = ...,
        *,
        for_layout_only=...
    ) -> BboxBase: ...
    def twinx(self) -> Axes: ...
    def twiny(self) -> Axes: ...
    def get_shared_x_axes(self) -> Grouper: ...
    def get_shared_y_axes(self) -> Grouper: ...



from scipy.special import xlogy as xlogy
from collections import namedtuple as namedtuple
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from .._typing import ArrayLike, Float
import numbers

import numpy as np


DistributionBoundary = ...


class ExponentialDispersionModel(metaclass=ABCMeta):
    def in_y_range(self, y: ArrayLike):
        ...

    @abstractmethod
    def unit_variance(self, y_pred: ArrayLike):
        ...

    @abstractmethod
    def unit_deviance(
        self, y: ArrayLike, y_pred: ArrayLike, check_input: bool = False
    ) -> ndarray:
        ...

    def unit_deviance_derivative(self, y: ArrayLike, y_pred: ArrayLike):
        ...

    def deviance(self, y: ArrayLike, y_pred: ArrayLike, weights: ArrayLike | int = 1):
        ...

    def deviance_derivative(
        self, y: ArrayLike, y_pred: ArrayLike, weights: ArrayLike | int = 1
    ):
        ...


class TweedieDistribution(ExponentialDispersionModel):
    def __init__(self, power: Float = 0) -> None:
        ...

    @property
    def power(self):
        ...

    @power.setter
    def power(self, power):
        ...

    def unit_variance(self, y_pred: ArrayLike):
        ...

    def unit_deviance(
        self, y: ArrayLike, y_pred: ArrayLike, check_input: bool = False
    ) -> ndarray:
        ...


class NormalDistribution(TweedieDistribution):
    def __init__(self) -> None:
        ...


class PoissonDistribution(TweedieDistribution):
    def __init__(self) -> None:
        ...


class GammaDistribution(TweedieDistribution):
    def __init__(self) -> None:
        ...


class InverseGaussianDistribution(TweedieDistribution):
    def __init__(self) -> None:
        ...


EDM_DISTRIBUTIONS: dict = ...



from ..utils.multiclass import type_of_target as type_of_target
from itertools import combinations as combinations
from ..utils import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
)

import numpy as np



"""
This type stub file was generated by pyright.
"""

from IPython.utils.decorators import undoc

"""
IO related utilities.
"""
class Tee:
    """A class to duplicate an output stream to stdout/err.

    This works in a manner very similar to the Unix 'tee' command.

    When the object is closed or deleted, it closes the original file given to
    it for duplication.
    """
    def __init__(self, file_or_name, mode=..., channel=...) -> None:
        """Construct a new Tee object.

        Parameters
        ----------
        file_or_name : filename or open filehandle (writable)
            File that will be duplicated
        mode : optional, valid mode for open().
            If a filename was give, open with this mode.
        channel : str, one of ['stdout', 'stderr']
        """
        ...
    
    def close(self): # -> None:
        """Close the file and restore the channel."""
        ...
    
    def write(self, data): # -> None:
        """Write data to both channels."""
        ...
    
    def flush(self): # -> None:
        """Flush both channels."""
        ...
    
    def __del__(self): # -> None:
        ...
    


def ask_yes_no(prompt, default=..., interrupt=...):
    """Asks a question and returns a boolean (y/n) answer.

    If default is given (one of 'y','n'), it is used if the user input is
    empty. If interrupt is given (one of 'y','n'), it is used if the user
    presses Ctrl-C. Otherwise the question is repeated until an answer is
    given.

    An EOF is treated as the default answer.  If there is no default, an
    exception is raised to prevent infinite loops.

    Valid answers are: y/yes/n/no (match is not case sensitive)."""
    ...

def temp_pyfile(src, ext=...):
    """Make a temporary python file, return filename and filehandle.

    Parameters
    ----------
    src : string or list of strings (no need for ending newlines if list)
        Source code to be written to the file.
    ext : optional, string
        Extension for the generated file.

    Returns
    -------
    (filename, open filehandle)
        It is the caller's responsibility to close the open file and unlink it.
    """
    ...

@undoc
def raw_print(*args, **kw): # -> None:
    """DEPRECATED: Raw print to sys.__stdout__, otherwise identical interface to print()."""
    ...

@undoc
def raw_print_err(*args, **kw): # -> None:
    """DEPRECATED: Raw print to sys.__stderr__, otherwise identical interface to print()."""
    ...




from collections import deque

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["tutte_polynomial", "chromatic_polynomial"]

def tutte_polynomial(G: Graph): ...
def chromatic_polynomial(G: Graph): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .visuals import *  # noqa
from .cameras import *  # noqa
from ..visuals.transforms import *  # noqa
from .widgets import *  # noqa
from .canvas import SceneCanvas as SceneCanvas  # noqa
from . import visuals as visuals  # noqa
from ..visuals import transforms as transforms  # noqa
from ..visuals import filters as filters  # noqa
from . import widgets as widgets  # noqa
from . import cameras as cameras  # noqa
from .node import Node as Node  # noqa



"""
This type stub file was generated by pyright.
"""

import ast
import collections
import operator
from typing import Any, Callable, Dict, Literal, NamedTuple, Set, TYPE_CHECKING, Tuple, Type, Union
from dataclasses import dataclass
from IPython.utils.decorators import undoc
from typing_extensions import Protocol

if TYPE_CHECKING or GENERATING_DOCUMENTATION:
    ...
else:
    ...
@undoc
class HasGetItem(Protocol):
    def __getitem__(self, key) -> None:
        ...
    


@undoc
class InstancesHaveGetItem(Protocol):
    def __call__(self, *args, **kwargs) -> HasGetItem:
        ...
    


@undoc
class HasGetAttr(Protocol):
    def __getattr__(self, key) -> None:
        ...
    


@undoc
class DoesNotHaveGetAttr(Protocol):
    ...


MayHaveGetattr = ...
@undoc
@dataclass
class EvaluationPolicy:
    """Definition of evaluation policy."""
    allow_locals_access: bool = ...
    allow_globals_access: bool = ...
    allow_item_access: bool = ...
    allow_attr_access: bool = ...
    allow_builtins_access: bool = ...
    allow_all_operations: bool = ...
    allow_any_calls: bool = ...
    allowed_calls: Set[Callable] = ...
    def can_get_item(self, value, item):
        ...
    
    def can_get_attr(self, value, attr):
        ...
    
    def can_operate(self, dunders: Tuple[str, ...], a, b=...): # -> None:
        ...
    
    def can_call(self, func): # -> None:
        ...
    


@undoc
@dataclass
class SelectivePolicy(EvaluationPolicy):
    allowed_getitem: Set[InstancesHaveGetItem] = ...
    allowed_getitem_external: Set[Tuple[str, ...]] = ...
    allowed_getattr: Set[MayHaveGetattr] = ...
    allowed_getattr_external: Set[Tuple[str, ...]] = ...
    allowed_operations: Set = ...
    allowed_operations_external: Set[Tuple[str, ...]] = ...
    _operation_methods_cache: Dict[str, Set[Callable]] = ...
    def can_get_attr(self, value, attr):
        ...
    
    def can_get_item(self, value, item): # -> None:
        """Allow accessing `__getiitem__` of allow-listed instances unless it was not modified."""
        ...
    
    def can_operate(self, dunders: Tuple[str, ...], a, b=...):
        ...
    


class _DummyNamedTuple(NamedTuple):
    """Used internally to retrieve methods of named tuple instance."""
    ...


class EvaluationContext(NamedTuple):
    locals: dict
    globals: dict
    evaluation: Literal["forbidden", "minimal", "limited", "unsafe", "dangerous"] = ...
    in_subscript: bool = ...


class _IdentitySubscript:
    """Returns the key itself when item is requested via subscript."""
    def __getitem__(self, key):
        ...
    


IDENTITY_SUBSCRIPT = ...
SUBSCRIPT_MARKER = ...
class GuardRejection(Exception):
    """Exception raised when guard rejects evaluation attempt."""
    ...


def guarded_eval(code: str, context: EvaluationContext): # -> None:
    """Evaluate provided code in the evaluation context.

    If evaluation policy given by context is set to ``forbidden``
    no evaluation will be performed; if it is set to ``dangerous``
    standard :func:`eval` will be used; finally, for any other,
    policy :func:`eval_node` will be called on parsed AST.
    """
    ...

BINARY_OP_DUNDERS: Dict[Type[ast.operator], Tuple[str]] = ...
COMP_OP_DUNDERS: Dict[Type[ast.cmpop], Tuple[str, ...]] = ...
UNARY_OP_DUNDERS: Dict[Type[ast.unaryop], Tuple[str, ...]] = ...
def eval_node(node: Union[ast.AST, None], context: EvaluationContext): # -> None:
    """Evaluate AST node in provided context.

    Applies evaluation restrictions defined in the context. Currently does not support evaluation of functions with keyword arguments.

    Does not evaluate actions that always have side effects:

    - class definitions (``class sth: ...``)
    - function definitions (``def sth: ...``)
    - variable assignments (``x = 1``)
    - augmented assignments (``x += 1``)
    - deletions (``del x``)

    Does not evaluate operations which do not return values:

    - assertions (``assert x``)
    - pass (``pass``)
    - imports (``import x``)
    - control flow:

        - conditionals (``if x:``) except for ternary IfExp (``a if x else b``)
        - loops (``for`` and `while``)
        - exception handling

    The purpose of this function is to guard against unwanted side-effects;
    it does not give guarantees on protection from malicious code execution.
    """
    ...

SUPPORTED_EXTERNAL_GETITEM = ...
BUILTIN_GETITEM: Set[InstancesHaveGetItem] = ...
dict_non_mutating_methods = ...
list_non_mutating_methods = ...
set_non_mutating_methods = ...
dict_keys: Type[collections.abc.KeysView] = ...
method_descriptor: Any = ...
module = ...
NUMERICS = ...
ALLOWED_CALLS = ...
BUILTIN_GETATTR: Set[MayHaveGetattr] = ...
BUILTIN_OPERATIONS = ...
EVALUATION_POLICIES = ...
__all__ = ["guarded_eval", "eval_node", "GuardRejection", "EvaluationContext", "_unbind_method"]
